# This file is part of Tulip (http://tulip.labri.fr)
#
# Authors: David Auber and the Tulip development Team
# from LaBRI, University of Bordeaux
#
# Tulip is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3
# of the License, or (at your option) any later version.
#
# Tulip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.

# -*- coding: utf-8 -*-

import os
import os.path
import sys
import platform

from tulip import tlp

# fix an ubuntu bug when using the nvidia driver,
# when the python interpreter dynamically loads the GL library,
# it ends up loading the mesa one instead of the nvidia one.
# The use of the tulipgui module through the classical python intrepreter
# was quite unusable because of that ...
# Apply the workaround proposed here :
# https://bugs.launchpad.net/ubuntu/+source/python-qt4/+bug/941826
if hasattr(os, 'uname'):
    if os.uname()[3].lower().find('ubuntu') != -1:
        import ctypes.util
        ctypes.CDLL(ctypes.util.find_library('GL'), ctypes.RTLD_GLOBAL)

_tulipGuiNativeLibsPath = '%s/native/' % os.path.dirname(__file__)
_tulipGuiNativePluginsPath = '%splugins' % _tulipGuiNativeLibsPath
_tulipGuiPluginsPath = '%s/plugins/' % os.path.dirname(__file__)

sys.path.append(_tulipGuiNativeLibsPath)

if platform.system() == 'Windows':
    os.environ['PATH'] = '%s;%s../../../;%s' % (
        _tulipGuiNativeLibsPath, _tulipGuiNativeLibsPath, os.environ['PATH'])

if not sys.argv[0] == 'tulip':
    # when the tulipgui module is installed from the Python Packaging Index
    # modify the TulipBitmapDir variable as it is different from
    # the default one
    bitmapDir = '%s/share/bitmaps/' % os.path.dirname(__file__)
    if os.path.isdir(bitmapDir):
        tlp.TulipBitmapDir = bitmapDir
        tlp.TulipPluginsPath = _tulipGuiNativePluginsPath
        tlp.TulipViewSettings.instance().setDefaultFontFile(
            '%sfont.ttf' % tlp.TulipBitmapDir)


import _tulipgui # noqa

sys.path.pop()


class tlpgui(_tulipgui.tlpgui):
    pass


def tulipguiExitFunc():
    import tulipgui
    tulipgui.tlpgui.runQtMainLoop()


# fix loading of Tulip plugins when the tulipgui module has been installed
# with the pip tool
if platform.system() == 'Linux' and os.path.exists(_tulipGuiNativePluginsPath):
    dlOpenFlagsBackup = sys.getdlopenflags()
    if sys.version_info < (3, 6):
        import DLFCN
        dlOpenFlags = DLFCN.RTLD_NOW | DLFCN.RTLD_GLOBAL
    else:
        dlOpenFlags = os.RTLD_NOW | os.RTLD_GLOBAL
    sys.setdlopenflags(dlOpenFlags)

tlp.loadTulipPluginsFromDir(_tulipGuiPluginsPath)

if (platform.system() == 'Linux'
        and os.path.exists(_tulipGuiNativePluginsPath)):
    sys.setdlopenflags(dlOpenFlagsBackup)

# Check if we are in script execution mode
# (sys.ps1 is not defined in that case)
# If so, register an exit callback that will run the Qt event loop
# if some widgets are still in a visible state
if not hasattr(sys, 'ps1') and not sys.flags.interactive:
    import atexit
    atexit.register(tulipguiExitFunc)

import signal # noqa
signal.signal(signal.SIGINT, signal.SIG_DFL)

__all__ = ['tlpgui']
__author__ = 'David Auber, and the Tulip development Team'
__license__ = 'LGPLv3'
__version__ = tlp.getTulipRelease()
__email__ = 'tulipdev@labri.fr'
__status__ = 'Production'



# This file is part of Tulip (http://tulip.labri.fr)
#
# Authors: David Auber and the Tulip development Team
# from LaBRI, University of Bordeaux
#
# Tulip is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3
# of the License, or (at your option) any later version.
#
# Tulip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.

import random
import os
import string
import sys
import unittest

from tulip import tlp

int_max = 32767
flt_max = 3.40282347e+38


def rand_int():
    return random.randint(-int_max, int_max)


def rand_float():
    return random.uniform(0, sys.float_info.max)


def rand_c_float():
    return random.uniform(0, flt_max)


def random_color():
    r = random.randint(0, 255)
    g = random.randint(0, 255)
    b = random.randint(0, 255)
    a = random.randint(0, 255)
    return tlp.Color(r, g, b, a)


def random_color_tuple():
    r = random.randint(0, 255)
    g = random.randint(0, 255)
    b = random.randint(0, 255)
    a = random.randint(0, 255)
    return (r, g, b, a)


def random_coord():
    return tlp.Coord(rand_c_float(), rand_c_float(), rand_c_float())


def random_coord_tuple():
    return (rand_c_float(), rand_c_float(), rand_c_float())


def random_coord_tuple_int():
    return (rand_int(), rand_int(), rand_int())


def random_size():
    return tlp.Size(rand_c_float(), rand_c_float(), rand_c_float())


def random_string():
    return ''.join([random.choice(string.ascii_letters + string.digits)
                    for _ in range(32)])


max_list_size = 30


def random_boolean_list(val=False):
    size = random.randint(max_list_size, max_list_size*2)
    return [val for i in range(size)]


def random_color_list():
    size = random.randint(1, max_list_size)
    return [random_color() for i in range(size)]


def random_color_tuple_list():
    size = random.randint(1, max_list_size)
    return [random_color_tuple() for i in range(size)]


def random_coord_list():
    size = random.randint(1, max_list_size)
    return [random_coord() for i in range(size)]


def random_coord_tuple_list():
    size = random.randint(1, max_list_size)
    return [random_coord_tuple() for i in range(size)]


def random_coord_tuple_int_list():
    size = random.randint(1, max_list_size)
    return [random_coord_tuple_int() for i in range(size)]


def random_double_list():
    size = random.randint(1, max_list_size)
    return [rand_float() for i in range(size)]


def random_int_list():
    size = random.randint(1, max_list_size)
    return [rand_int() for i in range(size)]


def random_size_list():
    size = random.randint(1, max_list_size)
    return [random_size() for i in range(size)]


def random_string_list():
    size = random.randint(1, max_list_size)
    return [random_string() for i in range(size)]


class TestGraphProperties(unittest.TestCase):

    def setUp(self):
        tlp.loadTulipPluginsFromDir(
            os.environ['TULIP_BUILD_DIR'] + '/plugins/import')
        self.prop_name = 'test_prop'
        self.prop = None
        self.graph = tlp.importGraph('Grid')
        self.n = self.graph.getRandomNode()
        self.e = self.graph.getRandomEdge()
        sg_nodes = [tlp.node(i) for i in range(self.graph.numberOfNodes()//2)]
        self.sub_graph = self.graph.inducedSubGraph(sg_nodes)

    def tearDown(self):
        if self.graph.existProperty(self.prop_name):
            self.graph.delLocalProperty(self.prop_name)
        self.prop = None
        self.graph = None
        self.n = None
        self.e = None

    def generic_property_test(self, prop_type, node_default_value,
                              edge_default_value, node_value, edge_value,
                              check_type_inference=True):
        # test some basic operations on property
        self.assertTrue(self.graph.existProperty(self.prop_name))
        self.assertTrue(
            isinstance(self.graph.getProperty(self.prop_name), prop_type))
        self.prop.setAllNodeValue(node_default_value)
        self.assertEqual(
            self.prop[self.graph.getRandomNode()], node_default_value)
        self.prop[self.n] = node_value
        self.prop[self.e] = edge_value
        self.assertEqual(self.prop[self.n], node_value)
        self.assertEqual(self.prop[self.e], edge_value)
        nodes_equal = list(self.prop.getNodesEqualTo(node_value))
        edges_equal = list(self.prop.getEdgesEqualTo(edge_value))
        self.assertEqual(len(nodes_equal), 1)
        self.assertEqual(len(edges_equal), 1)
        self.assertEqual(nodes_equal[0], self.n)
        self.assertEqual(edges_equal[0], self.e)

        self.prop.setAllNodeValue(node_default_value)
        self.prop.setValueToGraphNodes(node_value, self.sub_graph)
        for n in self.graph.getNodes():
            if self.sub_graph.isElement(n):
                self.assertEqual(self.prop[n], node_value)
            else:
                self.assertEqual(self.prop[n], node_default_value)

        self.prop.setAllEdgeValue(edge_default_value)
        self.prop.setValueToGraphEdges(edge_value, self.sub_graph)
        for e in self.graph.getEdges():
            if self.sub_graph.isElement(e):
                self.assertEqual(self.prop[e], edge_value)
            else:
                self.assertEqual(self.prop[e], edge_default_value)

        # ensure that an exception is correctly thrown when the property is
        # deleted from the C++ layer but there is still a reference to it
        # in the Python one
        self.graph.delLocalProperty(self.prop_name)
        self.assertFalse(self.graph.existProperty(self.prop_name))

        if sys.version_info >= (3,):
            assertRegex = self.assertRaisesRegex
        else:
            assertRegex = self.assertRaisesRegexp
        with assertRegex(
                RuntimeError, 'wrapped C/C\\+\\+ object .* has been deleted'):
            self.prop.setAllEdgeValue(edge_default_value)

        if not check_type_inference:
            return

        # test that the property can be automatically created by type inference
        self.graph[self.prop_name][self.graph.getRandomNode()] = node_value
        self.prop = self.graph[self.prop_name]
        self.assertTrue(
            isinstance(self.graph.getProperty(self.prop_name), prop_type))
        self.assertTrue(self.graph.existProperty(self.prop_name))

    def test_boolean_property(self):
        self.prop = self.graph.getBooleanProperty(self.prop_name)
        self.generic_property_test(
            tlp.BooleanProperty, True, False, False, True)

    def test_color_property(self):
        self.prop = self.graph.getColorProperty(self.prop_name)
        self.generic_property_test(
            tlp.ColorProperty, random_color(), random_color(), random_color(),
            random_color())

    def test_color_property_tuple(self):
        self.prop = self.graph.getColorProperty(self.prop_name)
        self.generic_property_test(
            tlp.ColorProperty, random_color_tuple(), random_color_tuple(),
            random_color_tuple(), random_color_tuple())

    def test_double_property(self):
        self.prop = self.graph.getDoubleProperty(self.prop_name)
        self.generic_property_test(
            tlp.DoubleProperty, rand_float(), rand_float(), rand_float(),
            rand_float())

    def test_integer_property(self):
        self.prop = self.graph.getIntegerProperty(self.prop_name)
        self.generic_property_test(
            tlp.IntegerProperty, rand_int(), rand_int(), rand_int(),
            rand_int())

    def test_layout_property(self):
        self.prop = self.graph.getLayoutProperty(self.prop_name)
        self.generic_property_test(
            tlp.LayoutProperty, random_coord(), random_coord_list(),
            random_coord(), random_coord_list())

    def test_layout_property_tuple(self):
        self.prop = self.graph.getLayoutProperty(self.prop_name)
        self.generic_property_test(
            tlp.LayoutProperty, random_coord_tuple(),
            random_coord_tuple_list(), random_coord_tuple(),
            random_coord_tuple_list())

    def test_layout_property_tuple_int(self):
        self.prop = self.graph.getLayoutProperty(self.prop_name)
        self.generic_property_test(
            tlp.LayoutProperty, random_coord_tuple_int(),
            random_coord_tuple_list(), random_coord_tuple_int(),
            random_coord_tuple_list(), check_type_inference=False)

    def test_size_property(self):
        self.prop = self.graph.getSizeProperty(self.prop_name)
        self.generic_property_test(
            tlp.SizeProperty, random_size(), random_size(), random_size(),
            random_size())

    def test_size_property_tuple_int(self):
        self.prop = self.graph.getSizeProperty(self.prop_name)
        self.generic_property_test(
            tlp.SizeProperty, random_coord_tuple_int(),
            random_coord_tuple_int(), random_coord_tuple_int(),
            random_coord_tuple_int(), check_type_inference=False)

    def test_string_property(self):
        self.prop = self.graph.getStringProperty(self.prop_name)
        self.generic_property_test(
            tlp.StringProperty, random_string(), random_string(),
            random_string(), random_string())

    def test_boolean_vector_property(self):
        self.prop = self.graph.getBooleanVectorProperty(self.prop_name)
        self.generic_property_test(
            tlp.BooleanVectorProperty, random_boolean_list(),
            random_boolean_list(), random_boolean_list(True),
            random_boolean_list(True))

    def test_color_vector_property(self):
        self.prop = self.graph.getColorVectorProperty(self.prop_name)
        self.generic_property_test(
            tlp.ColorVectorProperty, random_color_list(), random_color_list(),
            random_color_list(), random_color_list())

    def test_color_tuple_vector_property(self):
        self.prop = self.graph.getColorVectorProperty(self.prop_name)
        self.generic_property_test(
            tlp.ColorVectorProperty, random_color_tuple_list(),
            random_color_tuple_list(), random_color_tuple_list(),
            random_color_tuple_list())

    def test_coord_vector_property(self):
        self.prop = self.graph.getCoordVectorProperty(self.prop_name)
        self.generic_property_test(
            tlp.CoordVectorProperty, random_coord_list(), random_coord_list(),
            random_coord_list(), random_coord_list())

    def test_coord_tuple_vector_property(self):
        self.prop = self.graph.getCoordVectorProperty(self.prop_name)
        self.generic_property_test(
            tlp.CoordVectorProperty, random_coord_tuple_list(),
            random_coord_tuple_list(), random_coord_tuple_list(),
            random_coord_tuple_list())

    def test_coord_tuple_int_vector_property(self):
        self.prop = self.graph.getCoordVectorProperty(self.prop_name)
        self.generic_property_test(
            tlp.CoordVectorProperty, random_coord_tuple_int_list(),
            random_coord_tuple_int_list(), random_coord_tuple_int_list(),
            random_coord_tuple_int_list(), check_type_inference=False)

    def test_double_vector_property(self):
        self.prop = self.graph.getDoubleVectorProperty(self.prop_name)
        self.generic_property_test(
            tlp.DoubleVectorProperty, random_double_list(),
            random_double_list(), random_double_list(), random_double_list())

    def test_int_vector_property(self):
        self.prop = self.graph.getIntegerVectorProperty(self.prop_name)
        self.generic_property_test(
            tlp.IntegerVectorProperty, random_int_list(), random_int_list(),
            random_int_list(), random_int_list())

    def test_size_vector_property(self):
        self.prop = self.graph.getSizeVectorProperty(self.prop_name)
        self.generic_property_test(
            tlp.SizeVectorProperty, random_size_list(), random_size_list(),
            random_size_list(), random_size_list())

    def test_size_tuple_int_vector_property(self):
        self.prop = self.graph.getSizeVectorProperty(self.prop_name)
        self.generic_property_test(
            tlp.SizeVectorProperty, random_coord_tuple_int_list(),
            random_coord_tuple_int_list(), random_coord_tuple_int_list(),
            random_coord_tuple_int_list(), check_type_inference=False)

    def test_string_vector_property(self):
        self.prop = self.graph.getStringVectorProperty(self.prop_name)
        self.generic_property_test(
            tlp.StringVectorProperty, random_string_list(),
            random_string_list(), random_string_list(), random_string_list())



# This file is part of Tulip (http://tulip.labri.fr)
#
# Authors: David Auber and the Tulip development Team
# from LaBRI, University of Bordeaux
#
# Tulip is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3
# of the License, or (at your option) any later version.
#
# Tulip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.

# Tulip import plugin written in Python for importing the dependencies
# graph of a npm package (node package manager).
# The plugin simply parses the package.json files to extract the whole
# dependencies graph of a package. Be sure to have called 'npm install in
# the package directory first in order to get the complete dependencies graph

from tulip import tlp
import tulipplugins

import json
import os

npmPackageDirParamName = "npm package dir"

packageNode = {}


def parsePackageJsonForDependenciesGraph(npmPackageDir, graph,
                                         packageName=None):

    # get package.json file path
    if not packageName:
        # no package name provided as parameter, it is the root package
        packageJsonFilePath = '%s/package.json' % npmPackageDir
    else:
        # dependency package from the node_modules directory
        packageJsonFilePath = '%s/node_modules/%s/package.json' % (
            npmPackageDir, packageName)

    # package.json does not exist, abort
    if not os.path.exists(packageJsonFilePath):
        return

    # parse package.json file
    packageInfos = json.load(open(packageJsonFilePath))

    # create a string property that will store dependency type on edges
    dependencyType = graph.getStringProperty('dependencyType')

    # create the top level package node (entry point)
    if not packageName:
        packageName = packageInfos['name']
        packageNode[packageName] = graph.addNode()
        graph['viewLabel'][packageNode[packageName]] = packageName

    currentPackageNode = packageNode[packageName]

    # iterate over package dependencies
    for depType in ('dependencies', 'peerDependencies', 'devDependencies'):
        if depType in packageInfos:
            for dep in packageInfos[depType]:
                # create a node associated to the package if it does not
                # exist yet
                if dep not in packageNode:
                    packageNode[dep] = graph.addNode()
                    graph['viewLabel'][packageNode[dep]] = dep
                    # parse dependency package dependencies
                    parsePackageJsonForDependenciesGraph(
                        npmPackageDir, graph, dep)
                    parsePackageJsonForDependenciesGraph(
                        '%s/node_modules/%s' % (npmPackageDir, packageName),
                        graph, dep)
                # add an edge between the package and its depdency if it has
                # not already be created
                if not graph.hasEdge(currentPackageNode, packageNode[dep]):
                    depEdge = graph.addEdge(currentPackageNode,
                                            packageNode[dep])
                    dependencyType[depEdge] = depType


class NpmPackageDependenciesGraphImport(tlp.ImportModule):

    def __init__(self, context):
        tlp.ImportModule.__init__(self, context)
        self.addDirectoryParameter(
            npmPackageDirParamName, 'The root directory of the npm package')

    def icon(self):
        return '%s/npm.png' % os.path.dirname(__file__)

    def importGraph(self):
        npmPackageDir = self.dataSet[npmPackageDirParamName]
        # Check if the directory contains a npm package first
        rootPackageJson = '%s/package.json' % npmPackageDir
        if not os.path.exists(rootPackageJson):
            if self.pluginProgress:
                self.pluginProgress.setError(
                    ('Error : directory %s does not seem to contain a '
                     'npm package.') % npmPackageDir)
            return False

        # parse package dependencies graph
        parsePackageJsonForDependenciesGraph(npmPackageDir, self.graph)

        # apply a force directed algorithm to draw the graph
        self.graph.applyLayoutAlgorithm(
            'Fast Multipole Multilevel Embedder (OGDF)')

        return True


pluginDoc = """
Import the packages dependencies graph from a npm package.
Be sure to have called 'npm install' in the package directory first
in order to get the complete dependencies graph.
"""

# The line below does the magic to register the plugin to the plugin database
# and updates the GUI to make it accessible through the menus.
tulipplugins.registerPluginOfGroup(
    'NpmPackageDependenciesGraphImport', 'Npm package dependencies graph',
    'Antoine Lambert', '26/11/2015',
    pluginDoc, '1.0', 'Software')



# original module can be found on https://github.com/buzzfeed/pyh3

import math
import numpy as np

"""
Scaling factor for compute_radius(), compute_hyperbolic_area(),
compute_delta_theta() and compute_delta_phi().
"""
K = 2.0


"""
The 3D coordinate structure
"""


class Point4d(object):

    """
    The constructor for Point4d

    :param float x: the x coordinate for a node in cartesian space
    :param float y: the y coordinate for a node in cartesian space
    :param float z: the z coordinate for a node in cartesian space
    """

    def __init__(self, x=0.0, y=0.0, z=0.0, w=1.0):
        self.x = x
        self.y = y
        self.z = z
        self.w = w

    """
    Transform coordinate from spherical space to cartesian space

    :param float theta: the theta coordinate for a node in spherical space,
        polar angle
    :param float phi: the phi coordinate for a node in spherical space,
        elevation angle
    :param float r: the radius for a node in spherical space, radial distance
    """

    def sph_to_cart(self, theta, phi, r):
        self.x = r * math.sin(phi) * math.cos(theta)
        self.y = r * math.sin(phi) * math.sin(theta)
        self.z = r * math.cos(phi)

    """
    Add an offset to a node's coordinate in cartesian space

    :param Point4d offset: the offset to be added to current node's coordinate
    """

    def cart_offset(self, offset):
        self.x += offset.x
        self.y += offset.y
        self.z += offset.z

    """
    Translate node's coordinate in cartesian space by translation matrix

    :param Point4d offset: the offset to be added to current node's coordinate
    """

    def translate(self, offset):
        translation_matrix = np.array([[1, 0, 0, offset.x],
                                       [0, 1, 0, offset.y],
                                       [0, 0, 1, offset.z],
                                       [0, 0, 0, 1]])
        target = translation_matrix.dot(np.array([self.x, self.y,
                                                  self.z, self.w]))
        self.x, self.y, self.z, self.w = (target[0], target[1],
                                          target[2], target[3])

    """
    Transform node's coordinate in cartesian space by transformation matrix by
    spherical space coordinate values

    :param float theta: the theta coordinate for a node in spherical space,
        polar angle
    :param float phi: the phi coordinate for a node in spherical space,
        elevation angle
    """

    def coordinate_transformation(self, theta, phi):
        rotation_matrix = rotation_matrix_z(theta).dot(rotation_matrix_y(phi))
        target = rotation_matrix.dot(np.array([self.x, self.y,
                                               self.z, self.w]))
        self.x, self.y, self.z, self.w = (target[0], target[1],
                                          target[2], target[3])


"""
Compute the node's hemisphere radius by its hemisphere space reserved

:returns: return the node's radius as a float
"""


def compute_radius(H_p):
    return K * math.asinh(math.sqrt(H_p / (2 * math.pi * K * K)))


"""
Compute the parent node's hemisphere radius by its child's hemisphere radius
:returns: return the node's hemisphere space reservation as a float
"""


def compute_hyperbolic_area(radius):
    beta = 1.00
    return 2 * math.pi * (math.cosh(radius / K) - 1.0) * beta


"""
Compute the proper delta variant value for node's hemisphere placement,
similar to the space reservation the size of the node's hemisphere radius

:returns: return the node's half hemisphere space reservation as angle
    delta_theta in spherical space as a float number
"""


def compute_delta_theta(r, rp, phi):
    return math.atan(math.tanh(r / K) /
                     (math.sinh(rp / K) * math.sinh(phi)))


"""
Compute the proper phi variant value for a band of nodes' hemisphere placement,
similar to the space reservation the size of the largest node's hemisphere
radius of the whole band

:returns: return the nodes' max half hemisphere space reservation in the same
band as angle delta_phi in spherical space as a float number
"""


def compute_delta_phi(r, rp):
    return math.atan(math.tanh(r / K) / math.sinh(rp / K))


"""
The Klein metric for visualizing hyperbolic space: unusual uses of 4x4 matrices
by Phillips and Gunn
"""


def minkowski(x, y):
    return (x.x * y.x + x.y * y.y + x.z * y.z - x.w * y.w)


"""
Calculate 3D hyperbolic distance as a Klein metric
"""


def hyperbolic_distance(x, y):
    t1 = minkowski(x, y)
    t2 = minkowski(x, x)
    t3 = minkowski(y, y)
    return (2 * math.acosh(((t1 * t1) / (t2 * t3))**2))


"""
Rotation matrix around X axis

:params: the angle for rotating around X axis
:returns: return the rotation matrix around X axis by the given angle
"""


def rotation_matrix_x(angle):
    return np.array([[1, 0, 0, 0],
                     [0, math.cos(angle), -1 * math.sin(angle), 0],
                     [0, math.sin(angle), math.cos(angle), 0],
                     [0, 0, 0, 1]])


"""
Rotation matrix around Y axis

:params: the angle for rotating around Y axis
:returns: return the rotation matrix around Y axis by the given angle
"""


def rotation_matrix_y(angle):
    return np.array([[math.cos(angle), 0, math.sin(angle), 0],
                     [0, 1, 0, 0],
                     [-1 * math.sin(angle), 0, math.cos(angle), 0],
                     [0, 0, 0, 1]])


"""
Rotation matrix around Z axis

:params: the angle for rotating around Z axis
:returns: return the rotation matrix around X axis by the given angle
"""


def rotation_matrix_z(angle):
    return np.array([[math.cos(angle), -1 * math.sin(angle), 0, 0],
                     [math.sin(angle), math.cos(angle), 0, 0],
                     [0, 0, 1, 0],
                     [0, 0, 0, 1]])



# original module can be found on https://github.com/buzzfeed/pyh3

from .h3math import Point4d


"""
The node structure storing all attributes of a tree node.
"""


class Node(object):

    """
    The node constructor.

    :param int node_id: the id of node in the node lookup table in tree object
    :param int parent_id: the id of parent node in the node lookup table in
        tree object, default None
    :param int depth: the depth of this node in the tree, deault 0
    :param int tree_size: the subtree size of the node, ie. the number of
        nodes blow this node, default 1
    :param float radius: the node's hemisphere radius, ie. the distance to its
        children, default 0
    :param float area: the node's hemisphere area, default 0
    :param int band: the band which the node's hemisphere is placed in its
        parent's hemisphere, default -1
    :param float theta: the angle of the node's hemisphere rotating around Z
        axis in spherical space, default 0
    :param float phi: the angle between the node and Z axis in spherical space,
        default 0
    :param Point4d coord: the node's 3D coordinate in cartesisan space,
        default to Point4d(0,0,0,0)
    """

    def __init__(self, node_id, parent_id=None, depth=0, tree_size=1,
                 radius=0, area=0):
        self.node_id = node_id
        self.children = set()
        self.parent = parent_id
        self.depth = depth
        self.tree_size = tree_size
        self.radius = radius
        self.area = area
        self.band = -1
        self.theta = 0
        self.phi = 0
        self.coord = Point4d()

    def __repr__(self):
        return "<{0}>".format(self.node_id)



# This file is part of Tulip (http://tulip.labri.fr)
#
# Authors: David Auber and the Tulip development Team
# from LaBRI, University of Bordeaux
#
# Tulip is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3
# of the License, or (at your option) any later version.
#
# Tulip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.

from tulip import tlp
import tulipplugins

tulipGuiOk = True

try:
    from tulipgui import tlpgui
except Exception:
    tulipGuiOk = False


class H3LayoutHelper(tlp.Algorithm):
    def __init__(self, context):
        tlp.Algorithm.__init__(self, context)
        self.addFloatParameter(
            'layout scaling',
            'the scale factor to apply to the computed layout',
            '1000')

    def check(self):
        if not tlp.ConnectedTest.isConnected(self.graph):
            return (False, 'The graph must be connected')

        return (True, '')

    def run(self):

        scaling = 1000
        if self.dataSet:
            scaling = self.dataSet['layout scaling']

        params = tlp.getDefaultPluginParameters('H3', self.graph)
        params['layout scaling'] = scaling
        success, errmsg = self.graph.applyLayoutAlgorithm('H3', params)
        if not success:
            if self.pluginProgress:
                self.pluginProgress.setError(errmsg)
                return False

        self.graph['viewBorderWidth'].setAllNodeValue(0)
        self.graph['viewShape'].setAllNodeValue(tlp.NodeShape.Sphere)
        self.graph['viewTgtAnchorShape'].setAllEdgeValue(
            tlp.EdgeExtremityShape.Cone)

        for n in self.graph.getNodes():
            w = self.graph['viewSize'][n][0]
            h = self.graph['viewSize'][n][1]
            s = min(w, h)
            self.graph['viewSize'][n] = (s, s, s)

        if tulipGuiOk:
            for v in tlpgui.getViewsOfGraph(self.graph):
                if isinstance(v, tlpgui.NodeLinkDiagramComponent):
                    rp = v.getRenderingParameters()
                    rp.setEdge3D(True)
                    rp.setViewArrow(True)
                    rp.setLabelsAreBillboarded(True)
                    v.setRenderingParameters(rp)

        return True


pluginDoc = """
Enables to easily configure a H3 layout visualisation for a connected
quasi-hierarchical graph. As this is a 3d layout, some rendering setup
has to be done after the algorithm execution in order to get an aesthetic
rendering of it in Tulip. That plugin takes care of calling the H3 layout
algorithm, setting node shapes as sphere, setting edge extremity shapes to
cone and set appropriate rendering parameters for 3d layout visualization.
"""

# The line below does the magic to register the plugin into the plugin database
# and updates the GUI to make it accessible through the menus.
tulipplugins.registerPluginOfGroup(
    'H3LayoutHelper', 'H3 Layout Helper', 'Antoine Lambert', '01/09/2017',
    pluginDoc, '1.0', 'Misc')



# Make VCXProj
#
# November 2012
# Markus Chimani, markus.chimani@uni-jena.de
# Carsten Gutwenger, carsten.gutwenger@cs.tu-dortmund.de
#########################################################

# use Python 3 print function on both Python 2.x and Python 3.x
from __future__ import print_function

import os, sys, fnmatch

# Simple hack for using "configparser" on both Python 2 and 3
if sys.hexversion > 0x03000000:
	import configparser
else:
	import ConfigParser
	configparser = ConfigParser


class stuff:
	def __init__(self, ttag, tpath, tpats, tcommand, tfilter):
		self.tag, self.path, self.pats, self.command, self.filter = ttag, tpath, tpats, tcommand, tfilter

def bailout(msg):
	print(msg)
	print('Please use the original makeVCXProj.config as a template')
	sys.exit()

def loadConfig(sect, key, noError = False ):
	if config.has_option(sect, key):
		v = config.get(sect, key)
		print('   [', sect, ']', key, '=', v)
		return v
	else:
		if noError:
			return None
		else:
			bailout('Option "' + key + '" in section "' + sect + '" is missing')

def checkSolver(name, defSolver, extSolvers):
	if(name == defSolver):
		return True
	for s in extSolvers:
		s = s.strip()
		if(name == s):
			return True
	return False

def getLibs(libsConfig):
	libs = ''
	for str in libsConfig.split(';'):
		str = str.strip();
		if str != '':
			libs += str + ';'
	return libs


#########################################################
# LOAD CONFIGURATION

config = configparser.ConfigParser()

makecfg = 'makeVCXProj.config'
for x in sys.argv[1:]:
	if x[:7] == "config=":
		makecfg = x[7:]

print('Loading ' + makecfg + '...')

try:
	config.readfp( open(makecfg))
except IOError:
	bailout(makecfg + ' not found')

if not config.has_section('GENERAL'):
	bailout('Section "GENERAL" is missing')
if not config.has_section('OGDF'):
	bailout('Section "OGDF" is missing')
if not config.has_section('OGDF-TEST'):
	bailout('Section "OGDF-TEST" is missing')
if not config.has_section('COIN'):
	bailout('Section "COIN" is missing')

#########################################################
# CONFIGS

platformToolset = loadConfig('GENERAL', 'platformToolset')
windowsVersion = loadConfig('GENERAL', 'windowsVersion')
createSolution = loadConfig('GENERAL', 'createSolution').startswith('t')

# Filenames
filename_sln = loadConfig('GENERAL', 'solutionFile')
filename_vcxproj =  loadConfig('OGDF', 'projectFile')
filename_template = loadConfig('OGDF', 'templateFile')
filename_vcxfilters =  loadConfig('OGDF', 'projectFiltersFile')
filename_template_filters = loadConfig('OGDF', 'templateFiltersFile')
addDefines = ''
addIncludes = ''
addLibs = ''
addLibsWin32 = ''
addLibsWin32 = ''
addLibsX64 = ''
addLibsX64 = ''

# defines for auto generated config_autogen.h header file
config_defines = ''

createDLL = loadConfig('OGDF', 'DLL', True)

memoryManager = loadConfig('OGDF', 'memoryManager', True)
if memoryManager:
	config_defines += '#define ' + memoryManager + '\n'

ogdfOpenMP = 'false'
coinOpenMP = 'false'
ogdfEnableOpenMP = loadConfig('OGDF', 'OpenMP', True)
if ogdfEnableOpenMP and ogdfEnableOpenMP.startswith('t'):
	ogdfOpenMP = 'true'

useCoin = loadConfig('COIN', 'useCoin').startswith('t')
addOsiCpx = False
addOsiGrb = False
addLibsDebugWin32 = ''
addLibsReleaseWin32 = ''
addLibsDebugX64 = ''
addLibsReleaseX64 = ''
if useCoin:
	filename_coin_vcxproj =  loadConfig('COIN', 'projectFile')
	filename_coin_template = loadConfig('COIN', 'templateFile')
	filename_coin_vcxfilters =  loadConfig('COIN', 'projectFiltersFile')
	filename_coin_template_filters = loadConfig('COIN', 'templateFiltersFile')

	coinEnableOpenMP = loadConfig('COIN', 'OpenMP', True)
	if coinEnableOpenMP and coinEnableOpenMP.startswith('t'):
		coinOpenMP = 'true'

	defaultSolver   = loadConfig('COIN', 'defaultSolver')
	externalSolvers = loadConfig('COIN', 'externalSolvers').split(';')
	solverIncludes  = loadConfig('COIN', 'solverIncludes').split(';')

	addOsiCpx = checkSolver('CPX', defaultSolver, externalSolvers)
	addOsiGrb = checkSolver('GRB', defaultSolver, externalSolvers)

	config_defines += '#define USE_COIN\n'
	config_defines += '#define COIN_OSI_' + defaultSolver + '\n'

	for s in externalSolvers:
		s = s.strip();
		if s != '':
			config_defines += '#define OSI_' + s.strip() + '\n'

	for p in solverIncludes:
		p = p.strip()
		if p != '':
			addIncludes += p + ';'

	addLibs += 'coin.lib;'

	addLibsDebugWin32   += getLibs(loadConfig('COIN', 'solverLibs_win32_debug'))
	addLibsReleaseWin32 += getLibs(loadConfig('COIN', 'solverLibs_win32_release'))
	addLibsDebugX64     += getLibs(loadConfig('COIN', 'solverLibs_x64_debug'))
	addLibsReleaseX64   += getLibs(loadConfig('COIN', 'solverLibs_x64_release'))

linkSectionD32 = ''
linkSectionR32 = ''
linkSectionD64 = ''
linkSectionR64 = ''
if createDLL and createDLL.startswith('t'):
	libraryType = 'DynamicLibrary'
	config_defines += '#define OGDF_DLL\n'
	addDefines += 'OGDF_INSTALL'

	linkSectionBegin = '    <Link>\n\
	<AdditionalDependencies>';
	linkSectionEnd = 'kernel32.lib;user32.lib;gdi32.lib;winspool.lib;comdlg32.lib;advapi32.lib;shell32.lib;psapi.lib;%(AdditionalDependencies)</AdditionalDependencies>\n\
	<LinkTimeCodeGeneration>\n\
	</LinkTimeCodeGeneration>\n\
    <AdditionalLibraryDirectories>$(SolutionDir)$(Platform)\$(Configuration)\</AdditionalLibraryDirectories>\n\
    </Link>'

	linkSectionD32 = linkSectionBegin + addLibs + addLibsDebugWin32   + linkSectionEnd
	linkSectionR32 = linkSectionBegin + addLibs + addLibsReleaseWin32 + linkSectionEnd
	linkSectionD64 = linkSectionBegin + addLibs + addLibsDebugX64     + linkSectionEnd
	linkSectionR64 = linkSectionBegin + addLibs + addLibsReleaseX64   + linkSectionEnd

else:
	libraryType = 'StaticLibrary'

linkSectionProgramBegin = '    <Link>\n\
      <SubSystem>Console</SubSystem>\n\
      <AdditionalDependencies>';
linkSectionProgramEnd = 'ogdf.lib;kernel32.lib;user32.lib;gdi32.lib;winspool.lib;comdlg32.lib;advapi32.lib;shell32.lib;psapi.lib;%(AdditionalDependencies)</AdditionalDependencies>\n\
      <AdditionalLibraryDirectories>$(SolutionDir)$(Platform)\$(Configuration)\</AdditionalLibraryDirectories>\n\
    </Link>'
linkSectionProgramEndDebug = 'ogdf.lib;kernel32.lib;user32.lib;gdi32.lib;winspool.lib;comdlg32.lib;advapi32.lib;shell32.lib;psapi.lib;%(AdditionalDependencies)</AdditionalDependencies>\n\
      <AdditionalLibraryDirectories>$(SolutionDir)$(Platform)\$(Configuration)\</AdditionalLibraryDirectories>\n\
      <GenerateDebugInformation>true</GenerateDebugInformation>\n\
    </Link>'

linkSectionProgramD32 = linkSectionProgramBegin + addLibs + addLibsDebugWin32   + linkSectionProgramEndDebug
linkSectionProgramR32 = linkSectionProgramBegin + addLibs + addLibsReleaseWin32 + linkSectionProgramEnd
linkSectionProgramD64 = linkSectionProgramBegin + addLibs + addLibsDebugX64     + linkSectionProgramEndDebug
linkSectionProgramR64 = linkSectionProgramBegin + addLibs + addLibsReleaseX64   + linkSectionProgramEnd

	
filename_ogdf_test_vcxproj =  loadConfig('OGDF-TEST', 'projectFile')
filename_ogdf_test_template = loadConfig('OGDF-TEST', 'templateFile')
filename_ogdf_test_vcxfilters =  loadConfig('OGDF-TEST', 'projectFiltersFile')
filename_ogdf_test_template_filters = loadConfig('OGDF-TEST', 'templateFiltersFile')

print('Generating config_autogen.h ...')
config_autogen = open('include\\ogdf\\internal\\config_autogen.h','w')
config_autogen.write('//\n')
config_autogen.write('// This file has been automatically generated by makeVCXProj.py\n')
config_autogen.write('//\n')
config_autogen.write('// Do not change this file manually, instead reconfigure OGDF!\n')
config_autogen.write('\n')
config_autogen.write(config_defines)
config_autogen.write('\n#ifndef _WIN32_WINNT\n')
config_autogen.write('#define _WIN32_WINNT ' + windowsVersion + '\n')
config_autogen.write('#endif\n')
config_autogen.close()

addIncludes = addIncludes[:-1]
libraryTypeTag = '<<LIBRARYTYPETAG>>'
defineTag = '<<DEFINETAG>>'
includeTag = '<<INCLUDETAG>>'
linkTagD32 = '<<LINKTAGD32>>'
linkTagR32 = '<<LINKTAGR32>>'
linkTagD64 = '<<LINKTAGD64>>'
linkTagR64 = '<<LINKTAGR64>>'
filtersTag = '<<FTAG>>'
openMPTag = '<<OPENMPTAG>>'
toolsetTag = '<<TOOLSET>>'

# Params are:
# - Tag in template-File
# - Directory to start search & subfilters from
# - File Patterns
cppStuff = stuff( '<<CPPTAG>>', 'src\\ogdf', [ '*.c', '*.cpp' ], 'ClCompile', 'Source Files' )
hStuff = stuff( '<<HTAG>>', 'include\\ogdf', [ '*.h', '*.inc' ], 'ClInclude', 'Header Files' )
hLegacyStuff = stuff( '<<HLEGACYTAG>>', 'include\\ogdf_legacy', [ '*.h' ], 'ClInclude', 'Header Files Legacy' )
ogdfStuff = [ cppStuff, hStuff, hLegacyStuff ]

cppCoinStuff = stuff( '<<CPPTAG>>', 'src\\coin', [ '*.c', '*.cpp' ], 'ClCompile', 'Source Files' )
hCoinStuff = stuff( '<<HTAG>>', 'include\\coin', [ '*.h', '*.hpp' ], 'ClInclude', 'Header Files' )
coinStuff = [ cppCoinStuff, hCoinStuff ]

cppTestStuff = stuff( '<<CPPTAG>>', 'test', [ '*.c', '*.cpp' ], 'ClCompile', 'Source Files' )
hTestStuff = stuff( '<<HTAG>>', 'test', [ '*.h', '*.hpp' ], 'ClInclude', 'Header Files' )
testStuff = [ cppTestStuff, hTestStuff ]

#########################################################
#########################################################
## only code below...

# just the def. nothing happens yet
def Walk( curdir, pats, command ):
	names = os.listdir( curdir)
	names.sort()

	for name in names:
		# OGDF ignores
		if name.startswith('.') or name.startswith('_') or (name=='legacy' and not includeLegacyCode):
			continue
		# COIN ignores
		if (name == 'OsiCpxSolverInterface.cpp' or name == 'OsiCpxSolverInterface.hpp') and not addOsiCpx:
			continue
		if (name == 'OsiGrbSolverInterface.cpp' or name == 'OsiGrbSolverInterface.hpp') and not addOsiGrb:
			continue

		outpath = curdir + '\\' + name
		fullname = os.path.normpath(outpath)

		if os.path.isdir(fullname) and not os.path.islink(fullname):
			if name != 'abacus' or useCoin == True:
				Walk( outpath, pats, command)
		else:
			for pat in pats:
				if fnmatch.fnmatch(name, pat):
					vcxproj.write('    <' + command + ' Include="' + outpath + '" />\n')

def WalkFilterFiles( curdir, pats, command, filter ):
	names = os.listdir( curdir)
	names.sort()

	for name in names:
		# OGDF ignores
		if name.startswith('.') or name.startswith('_') or (name=='legacy' and not includeLegacyCode):
			continue
		# COIN ignores
		if (name == 'OsiCpxSolverInterface.cpp' or name == 'OsiCpxSolverInterface.h') and not addOsiCpx:
			continue
		if (name == 'OsiGrbSolverInterface.cpp' or name == 'OsiGrbSolverInterface.h') and not addOsiGrb:
			continue

		outpath = curdir + '\\' + name
		fullname = os.path.normpath(outpath)

		if os.path.isdir(fullname) and not os.path.islink(fullname):
			WalkFilterFiles( outpath, pats, command, filter + '\\' + name )
		else:
			for pat in pats:
				if fnmatch.fnmatch(name, pat):
					vcxfilters.write('    <' + command + ' Include="' + outpath + '">\n')
					vcxfilters.write('      <Filter>' + filter + '</Filter>\n')
					vcxfilters.write('    </' + command + '>\n')

def WalkFilters( curdir, filter ):
	names = os.listdir( curdir)
	names.sort()

	for name in names:
		if name.startswith('.') or name.startswith('_') or (name=='legacy' and not includeLegacyCode):
			continue

		outpath = curdir + '\\' + name
		fullname = os.path.normpath(outpath)

		if os.path.isdir(fullname) and not os.path.islink(fullname):
			if name != 'abacus' or useCoin == True:
				filtername = filter + '\\' + name
				vcxfilters.write('    <Filter Include="' + filtername + '">\n')
				vcxfilters.write('    </Filter>\n')
				WalkFilters( outpath, filtername )


##########################################
## Main below...

print('Generating OGDF VCXProj...', end='')

includeLegacyCode = 0;
if len(sys.argv)>1 and sys.argv[1]=='legacy':
	includeLegacyCode = 1
	print('(including legacy code)...'),

vcxproj = open(filename_vcxproj,'w')
template = open(filename_template)

check = 0
for line in template:
	if check < len(ogdfStuff) and line.find(ogdfStuff[check].tag) > -1:
		if (ogdfStuff[check].tag!='<<HLEGACYTAG>>' or includeLegacyCode):
			Walk(ogdfStuff[check].path, ogdfStuff[check].pats, ogdfStuff[check].command)
		check = check + 1
	elif line.find(libraryTypeTag) > -1:
		vcxproj.write(line.replace(libraryTypeTag,libraryType,1))
	elif line.find(openMPTag) > -1:
		vcxproj.write(line.replace(openMPTag,ogdfOpenMP,1))
	elif line.find(defineTag) > -1:
		vcxproj.write(line.replace(defineTag,addDefines,1))
	elif line.find(includeTag) > -1:
		vcxproj.write(line.replace(includeTag,addIncludes,1))
	elif line.find(linkTagD32) > -1:
		vcxproj.write(line.replace(linkTagD32,linkSectionD32,1))
	elif line.find(linkTagR32) > -1:
		vcxproj.write(line.replace(linkTagR32,linkSectionR32,1))
	elif line.find(linkTagD64) > -1:
		vcxproj.write(line.replace(linkTagD64,linkSectionD64,1))
	elif line.find(linkTagR64) > -1:
		vcxproj.write(line.replace(linkTagR64,linkSectionR64,1))
	elif line.find(toolsetTag) > -1:
		vcxproj.write(line.replace(toolsetTag,platformToolset,1))
	else:
		vcxproj.write(line)

template.close()
vcxproj.close()

# Creation of filters file...

vcxfilters = open(filename_vcxfilters,'w')
template_filters = open(filename_template_filters)

check = 0
for line in template_filters:
	if check < len(ogdfStuff) and line.find(ogdfStuff[check].tag) > -1:
		if (ogdfStuff[check].tag!='<<HLEGACYTAG>>' or includeLegacyCode):
			WalkFilterFiles(ogdfStuff[check].path, ogdfStuff[check].pats, ogdfStuff[check].command, ogdfStuff[check].filter)
		check = check + 1
	elif line.find(filtersTag) > -1:
		for s in ogdfStuff:
			if (s.tag!='<<HLEGACYTAG>>' or includeLegacyCode):
				WalkFilters(s.path, s.filter)
	else:
		vcxfilters.write(line)

template_filters.close()
vcxfilters.close()

print('done')

if useCoin:
	print ('Generating COIN VCXProj...', end='')

	vcxproj = open(filename_coin_vcxproj,'w')
	template = open(filename_coin_template)

	check = 0
	for line in template:
		if check < len(coinStuff) and line.find(coinStuff[check].tag) > -1:
			Walk(coinStuff[check].path, coinStuff[check].pats, coinStuff[check].command)
			check = check + 1
		elif line.find(openMPTag) > -1:
			vcxproj.write(line.replace(openMPTag,coinOpenMP,1))
		elif line.find(includeTag) > -1:
			vcxproj.write(line.replace(includeTag,addIncludes,1))
		elif line.find(toolsetTag) > -1:
			vcxproj.write(line.replace(toolsetTag,platformToolset,1))
		else:
			vcxproj.write(line)

	template.close()
	vcxproj.close()

	# Creation of filters file...

	vcxfilters = open(filename_coin_vcxfilters,'w')
	template_filters = open(filename_coin_template_filters)

	check = 0
	for line in template_filters:
		if check < len(coinStuff) and line.find(coinStuff[check].tag) > -1:
			WalkFilterFiles(coinStuff[check].path, coinStuff[check].pats, coinStuff[check].command, coinStuff[check].filter)
			check = check + 1
		elif line.find(filtersTag) > -1:
			for s in coinStuff:
				WalkFilters(s.path, s.filter)
		else:
			vcxfilters.write(line)

	template_filters.close()
	vcxfilters.close()

	print ('done')

print ('Generating OGDF-TEST VCXProj...', end='')

vcxproj = open(filename_ogdf_test_vcxproj,'w')
template = open(filename_ogdf_test_template)

check = 0
for line in template:
	if check < len(testStuff) and line.find(testStuff[check].tag) > -1:
		Walk(testStuff[check].path, testStuff[check].pats, testStuff[check].command)
		check = check + 1
	elif line.find(includeTag) > -1:
		vcxproj.write(line.replace(includeTag,addIncludes,1))
	elif line.find(linkTagD32) > -1:
		vcxproj.write(line.replace(linkTagD32,linkSectionProgramD32,1))
	elif line.find(linkTagR32) > -1:
		vcxproj.write(line.replace(linkTagR32,linkSectionProgramR32,1))
	elif line.find(linkTagD64) > -1:
		vcxproj.write(line.replace(linkTagD64,linkSectionProgramD64,1))
	elif line.find(linkTagR64) > -1:
		vcxproj.write(line.replace(linkTagR64,linkSectionProgramR64,1))
	elif line.find(toolsetTag) > -1:
		vcxproj.write(line.replace(toolsetTag,platformToolset,1))
	else:
		vcxproj.write(line)

template.close()
vcxproj.close()

# Creation of filters file...

vcxfilters = open(filename_ogdf_test_vcxfilters,'w')
template_filters = open(filename_ogdf_test_template_filters)

check = 0
for line in template_filters:
	if check < len(testStuff) and line.find(testStuff[check].tag) > -1:
		WalkFilterFiles(testStuff[check].path, testStuff[check].pats, testStuff[check].command, testStuff[check].filter)
		check = check + 1
	elif line.find(filtersTag) > -1:
		for s in testStuff:
			WalkFilters(s.path, s.filter)
	else:
		vcxfilters.write(line)

template_filters.close()
vcxfilters.close()

print ('done')

if createSolution:
	print ('Generating Solution...', end='')

	GUID_sln = '{0F7C385F-D08C-494E-8715-70CD736C75B2}'
	GUID_ogdf = '{7801D1BE-E2FE-476B-A4B4-5D27F387F479}'
	GUID_coin = '{FB212DCC-D374-430B-B594-5CEC25BC7A75}'
	GUID_ogdf_test = '{278A54D6-588A-4110-9E44-DA24DB9D5E85}'

	Configurations = [ 'Debug', 'Release' ]
	Platforms = [ 'Win32', 'x64' ]

	sln = open(filename_sln, 'w')
	sln.write('Microsoft Visual Studio Solution File, Format Version 11.00\n')
	
	sln.write('Project("' + GUID_sln + '") = "ogdf-test", "' + filename_ogdf_test_vcxproj + '", "' + GUID_ogdf_test + '"\n')
	sln.write('\tProjectSection(ProjectDependencies) = postProject\n')
	sln.write('\t\t' + GUID_ogdf + ' = ' + GUID_ogdf + '\n')
	if useCoin:
		sln.write('\t\t' + GUID_coin + ' = ' + GUID_coin + '\n')
	sln.write('\tEndProjectSection\n')
	sln.write('EndProject\n')

	sln.write('Project("' + GUID_sln + '") = "ogdf", "' + filename_vcxproj + '", "' + GUID_ogdf + '"\n')
	if createDLL and createDLL.startswith('t') and useCoin:
		sln.write('\tProjectSection(ProjectDependencies) = postProject\n')
		sln.write('\t\t' + GUID_coin + ' = ' + GUID_coin + '\n')
		sln.write('\tEndProjectSection\n')
	sln.write('EndProject\n')

	if useCoin:
		sln.write('Project("' + GUID_sln + '") = "coin", "' + filename_coin_vcxproj + '", "' + GUID_coin + '"\n')
		sln.write('EndProject\n')

	sln.write('Global\n')
	sln.write('\tGlobalSection(SolutionConfigurationPlatforms) = preSolution\n')
	sln.write('\t\tDebug|Win32 = Debug|Win32\n')
	sln.write('\t\tDebug|x64 = Debug|x64\n')
	sln.write('\t\tRelease|Win32 = Release|Win32\n')
	sln.write('\t\tRelease|x64 = Release|x64\n')
	sln.write('\tEndGlobalSection\n')
	sln.write('\tGlobalSection(ProjectConfigurationPlatforms) = postSolution\n')

	if useCoin:
		projectGUIDs = [ GUID_ogdf, GUID_coin, GUID_ogdf_test ]
	else:
		projectGUIDs = [ GUID_ogdf, GUID_ogdf_test ]
	for g in projectGUIDs:
		for c in Configurations:
			for p in Platforms:
				sln.write('\t\t' + g + '.' + c + '|' + p + '.ActiveCfg' + ' = ' + c + '|' + p + '\n')
				sln.write('\t\t' + g + '.' + c + '|' + p + '.Build.0' + ' = ' + c + '|' + p + '\n')
	sln.write('\tEndGlobalSection\n')
	sln.write('\tGlobalSection(SolutionProperties) = preSolution\n')
	sln.write('\t\tHideSolutionNode = FALSE\n')
	sln.write('\tEndGlobalSection\n')
	sln.write('EndGlobal\n')

	sln.close()
	print ('done')



# original module can be found on https://github.com/buzzfeed/pyh3

import logging
import math
from collections import deque
from .node import Node
from operator import itemgetter
from .h3math import (
    compute_radius, compute_hyperbolic_area,
    compute_delta_phi, compute_delta_theta
)

"""
Customized exception for invalid edge input.
"""


class InvalidArgument(Exception):
    pass


"""
The tree structure storing all nodes and edges, and also provide easy
node lookup.
"""


class Tree(object):

    """
    The Tree class is the entry point for this application. It's only argument
    is an iterable edgelist. This can be a generator, list, tuple, etc.
    Each edge should be of the format (parent_id, child_id). They should be
    int types (long is fine) and they should be consecutive. The edgelist must
    form a single connected Tree. If that is not the case you'll need to prune
    your graph to make this the case. A common operation for displaying
    otherwise cyclic graphs is to simply plot the minimum spanning tree.

    :param edges: a tuple for a tree edge as (parent, child). This is required.
    :type edges: iterable(tuple(int, int))
    """
    def __init__(self, edges=None):
        self.nodes = {}
        self.height = 0
        self.root = None

        for parent_id, child_id in edges:
            self.insert_edge(child_id, parent_id)

        edges = None  # Release that memory.

        for id, node in self.nodes.items():
            if node.parent is None:
                self.root = node
                break

        self.__label_node_generations()
        self.set_subtree_radius()
        self.set_subtree_size()
        self.sort_children_by_radius()
        self.set_placement()

    def insert_edge(self, parent_id, child_id):
        """
        Insert edge(parent, child) pair into the tree. No assumptions are made
        about the structure except that a Node can have exactly 1 parent,
        and each child can only occur once. Otherwise; cyclic graphs can be
        input. This will probably cause your layout to run forever.

        :param int parent_id: The id of the parent node.
        :param int child_id: The id of the child node.

        :returns: A tuple of the parent and child nodes as Node objects.
        """
        parent = self.nodes.get(parent_id, Node(parent_id))
        child = self.nodes.get(child_id, Node(child_id))

        child.parent = parent
        parent.children.add(child)

        self.nodes[parent_id] = parent
        self.nodes[child_id] = child

        return parent, child

    def get_leaf_nodes(self):
        """
        Gets the node ids for all leaf nodes in the Tree.

        :returns: A generator of all the leaf nodes in the Tree
        """
        for node_id, node in self.nodes.items():
            if not node.children:
                yield node.node_id

    def print_tree(self):
        """
        Print the tree sorted by depth to log, including the following
        parameters. The tree is traversed in a breath-first-search.
        """
        current_generation = deque([self.root.node_id])
        next_generation = True
        while next_generation:
            next_generation = deque()
            while current_generation:
                node_id = current_generation.popleft()
                logging.info(
                    ('{0}, parent: {1}, depth: {2}, #children: {3}, size: {4},'
                     ' radius: {5}, area: {6}')
                    .format(node_id,
                            self.nodes[node_id].parent,
                            self.nodes[node_id].depth,
                            len(self.nodes[node_id].children),
                            self.nodes[node_id].tree_size,
                            self.nodes[node_id].radius,
                            self.nodes[node_id].area))
                for child in self.nodes[node_id].children:
                    next_generation.append(child.node_id)
            current_generation = next_generation

    def __label_node_generations(self, depth=0):
        """
        Decorate each node with it's depth in the tree.

        :param int depth: the initial depth value for the root, default 0
        """
        current_generation = deque([self.root.node_id])
        next_generation = True
        while next_generation:
            next_generation = deque()
            while current_generation:
                node_id = current_generation.popleft()
                self.nodes[node_id].depth = depth
                for child in self.nodes[node_id].children:
                    next_generation.append(child.node_id)
            depth += 1
            current_generation = next_generation
        self.height = depth - 1

    def set_subtree_radius(self):
        """
        Set the node's hemisphere radius and also its distance from its
        children. The radius is calculated recursively from the leaf nodes,
        with a unit hemisphere size, tracing back to the root. The area
        calculation is an approximation from a disc of the bottom of a child
        hemisphere to a spherical cap. The recursion requires tracing the tree
        from the last generation to root so that all the nodes radii have been
        calculated before their parent's radius is calculated. As the
        hemisphere sizes are tightly calculated but placing them loosely to
        the parent hemisphere, the space reservation is 7.2 times of the actual
        size of a child hemisphere.

        """
        leaf_nodes = self.get_leaf_nodes()
        outermost_non_leaf = set()
        for n in leaf_nodes:
            self.nodes[n].radius = compute_radius(0.0025)
            logging.info('leaf node {0}, parent {1}, radius {2}'
                         .format(n, self.nodes[n].parent,
                                 self.nodes[n].radius))
            outermost_non_leaf.add(self.nodes[n].parent.node_id)
        depth = self.height - 1
        current_generation = deque(list(
            set(n.node_id for n in outermost_non_leaf
                if self.nodes[n].parent is not None
                if self.nodes[self.nodes[n].parent.node_id].depth == depth)
        ))
        previous_generation = True
        while previous_generation:
            previous_generation = deque()
            while current_generation:
                n = current_generation.popleft()
                if self.nodes[n].area == 0:  # avoid duplicate parents
                    if self.nodes[n].parent is not None:
                        previous_generation.append(
                            self.nodes[n].parent.node_id)
                    for child in self.nodes[n].children:
                        self.nodes[n].area += 7.2 * compute_hyperbolic_area(
                            self.nodes[child.node_id].radius)
                        logging.info(('node {0}, child {1}, child_area+ {2},'
                                      ' radius {3}, area {4}')
                                     .format(n, child, compute_hyperbolic_area(
                                        self.nodes[child.node_id].radius),
                                        self.nodes[child.node_id].radius,
                                        self.nodes[child.node_id].area))
                    self.nodes[n].radius = compute_radius(self.nodes[n].area)
                    logging.info('---> node {0}, radius {1}, area {2}'
                                 .format(n, self.nodes[n].radius,
                                         self.nodes[n].area))
            for n in outermost_non_leaf:
                if n is not None:
                    if self.nodes[n].depth == depth:
                        previous_generation.append(n)
            depth -= 1
            current_generation = deque(list(set(previous_generation)))

    def set_subtree_size(self):
        """
        Set the subtree size by the number of nodes in its subtree.
        """
        leaf_nodes = self.get_leaf_nodes()
        depth = self.height
        current_generation = deque(list(n for n in leaf_nodes
                                        if self.nodes[n].depth == depth))
        previous_generation = True
        while previous_generation:
            depth -= 1
            previous_generation = deque()
            while current_generation:
                n = current_generation.popleft()
                if self.nodes[n].parent is not None:
                    previous_generation.append(self.nodes[n].parent.node_id)
                    self.nodes[self.nodes[n].parent.node_id].tree_size += \
                        self.nodes[n].tree_size
            for n in leaf_nodes:
                if self.nodes[n].depth == depth:
                    previous_generation.append(n)
            current_generation = deque(list(set(previous_generation)))

    def sort_children_by_radius(self):
        """
        Sort the nodes in decreasing order in the same depth by their radii,
        in place sort is used. The tree is traversed in a breath-first-search.
        """
        depth = 0
        current_generation = deque([self.root.node_id])
        next_generation = True
        while next_generation:
            next_generation = deque()
            while current_generation:
                node_id = current_generation.popleft()
                for child in self.nodes[node_id].children:
                    next_generation.append(child.node_id)
                child_size_pair = [[child, self.nodes[child.node_id].radius]
                                   for child in self.nodes[node_id].children]
                child_size_pair.sort(key=itemgetter(1), reverse=True)
                if child_size_pair:
                    self.nodes[node_id].children = list(
                        zip(*child_size_pair))[0]
            depth += 1
            current_generation = next_generation

    def sort_children_by_tree_size(self):
        """
        Sort the nodes in decreasing order in the same depth by their number
        of nodes in subtree, in place sort is used. This is an alternative
        option to sort the tree before placing the nodes on the hemisphere.
        The original H3 algorithm set leaf node radius as math:: N / alpha
        so the nodes with many sibilings can have a larger radius and nodes
        with a lot of children.
        """
        depth = 0
        current_generation = deque([self.root])
        next_generation = True
        while next_generation:
            next_generation = deque()
            while current_generation:
                node_id = current_generation.popleft()
                for child in self.nodes[node_id].children:
                    next_generation.append(child)
                child_size_pair = [[child, self.nodes[child].tree_size]
                                   for child in self.nodes[node_id].children]
                child_size_pair.sort(key=itemgetter(1), reverse=True)
                if child_size_pair:
                    self.nodes[node_id].children = list(
                        zip(*child_size_pair))[0]
            depth += 1
            current_generation = next_generation

    def set_placement(self):
        """
        Placing the hemispheres on the root hemisphere. Start from the pole,
        placing the largest child hemisphere and then placing smaller
        hemispheres around the pole. When the hemispheres fully filled
        one band, start placing on the next band until fully filled again.
        Node:
            - Placing the 1st hemisphere and its phi is zero, could lead to a
              ZeroDivisionError exception
              so we set its value as a very small number (0.000001)
            - Don't forget to reserve space for the other half of the
              hemisphere, both the right half and the lower half.
              Add theta by delta_theta after placing each node. Add phi by the
              max delta_phi in the last band before placing hemispheres to the
              next band
            - Each subtree is independent from each other, so if the new node
              has a different parent node than the previous parent node, we
              know this node is in a different subtree and initialize phi,
              theta, dealta_theta and band and placing nodes all over again.
        """
        depth = 0
        current_generation = deque(
            [n.node_id for n in self.nodes[self.root.node_id].children])
        next_generation = True
        last_parent = self.root
        while next_generation:
            next_generation = deque()
            phi, theta, delta_theta, band = 0.000001, 0., 0., 1
            last_max_phi = 0  # span phi before jumping to the next band
            while current_generation:
                node = current_generation.popleft()
                # same gen, diff parent
                if self.nodes[node].parent != last_parent:
                    last_parent = self.nodes[node].parent
                    phi, theta, delta_theta, band = 0.000001, 0., 0., 1
                rp = self.nodes[self.nodes[node].parent.node_id].radius
                try:
                    if phi == 0.000001:  # first child of root
                        phi += compute_delta_phi(self.nodes[node].radius, rp)
                        self.nodes[node].band = 0
                    else:
                        delta_theta = compute_delta_theta(
                            self.nodes[node].radius, rp, phi)
                        if (theta + delta_theta) <= 2 * math.pi:
                            theta += delta_theta
                            if last_max_phi:
                                last_max_phi = compute_delta_phi(
                                    self.nodes[node].radius, rp)
                                phi += compute_delta_phi(
                                    self.nodes[node].radius, rp)
                        else:
                            band += 1
                            theta = delta_theta
                            phi += last_max_phi + compute_delta_phi(
                                self.nodes[node].radius, rp)
                            last_max_phi = 0
                        self.nodes[node].band = band
                        self.nodes[node].theta = theta
                        self.nodes[node].phi = phi
                except ZeroDivisionError as e:
                    logging.error(('{0}\n node {1}, radius={2}, rp={3}, '
                                   'phi={4}, parent={5}')
                                  .format(e, node, self.nodes[node].radius, rp,
                                          phi, self.nodes[node].parent))
                self.nodes[node].coord.sph_to_cart(
                    self.nodes[node].theta,
                    self.nodes[node].phi,
                    self.nodes[self.nodes[node].parent.node_id].radius)
                if self.nodes[node].parent != self.root.node_id:
                    self.nodes[node].coord.coordinate_transformation(
                        self.nodes[self.nodes[node].parent.node_id].theta,
                        self.nodes[self.nodes[node].parent.node_id].phi)
                    self.nodes[node].coord.cart_offset(
                        self.nodes[self.nodes[node].parent.node_id].coord)
                logging.info(('node {0}, radius {1},  band {2}, theta {3}, '
                              'phi {4}')
                             .format(node, self.nodes[node].radius,
                                     self.nodes[node].band,
                                     self.nodes[node].theta,
                                     self.nodes[node].phi))
                logging.info('node {0}, x {1}, y {2}, z {3}, w {4}'
                             .format(node, self.nodes[node].coord.x,
                                     self.nodes[node].coord.y,
                                     self.nodes[node].coord.z,
                                     self.nodes[node].coord.w))
                # reserve space for the other half sphere
                theta += delta_theta
                for child in self.nodes[node].children:
                    next_generation.append(child.node_id)
            depth += 1
            current_generation = next_generation

    def get_coordinates(self):
        """
        Returns a generator of all the coordinates for this plot.
        This can be used to build SVGs or whatever.

        :return: A generator of tuples of (x, y, z) representing the position
            of each node in 3-d space.
        """
        current_generation = deque([self.root.node_id])
        next_generation = True
        while next_generation:
            next_generation = deque()
            while current_generation:
                node_id = current_generation.popleft()
                yield (self.nodes[node_id].coord.x,
                       self.nodes[node_id].coord.y,
                       self.nodes[node_id].coord.z)
                for child in self.nodes[node_id].children:
                    next_generation.append(child.node_id)
            current_generation = next_generation



# This file is part of Tulip (http://tulip.labri.fr)
#
# Authors: David Auber and the Tulip development Team
# from LaBRI, University of Bordeaux
#
# Tulip is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3
# of the License, or (at your option) any later version.
#
# Tulip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.

# Simple Python plugin to ease the configuration of a Treemap visualization
# in the main Tulip GUI. As the treemap layout is different from classical
# node link diagram representation, some visual properties setup has to be
# done in order to get an aesthetic visualization of it in the main Tulip GUI.
# Notably, edge colors must be set to full transparency to avoid rendering
# artifacts, node shapes must be set to square, node border colors must be
# adjusted (everything but black) and node border widths must be visible
# (different from 0).

from tulip import tlp
import tulipplugins


class SquarifiedTreeMapHelper(tlp.Algorithm):

    def __init__(self, context):
        tlp.Algorithm.__init__(self, context)

        self.addNumericPropertyParameter(
            'metric',
            ('An optional metric used to estimate the size allocated '
             'to each node'), '', False)

        self.addFloatParameter(
            'aspect ratio',
            ('The aspect ratio (height/width) for the rectangle corresponding'
             ' to the root node'), '1')

        self.addBooleanParameter(
            'treemap type',
            ('If true, use original Treemaps (B. Shneiderman) otherwise use'
             'Squarified Treemaps (J. J. van Wijk)'), 'false')

        self.addColorParameter(
            'border color',
            'The border color that will be applied to all treemap nodes',
            '(255, 255, 255, 255)')

        self.addLayoutPropertyParameter(
            'layout', 'The output treemap layout', 'viewLayout',
            True, False, True)

        self.addSizePropertyParameter(
            'sizes', 'The output treemap sizes', 'viewSize', True, False, True)

        self.addIntegerPropertyParameter(
            'shapes', 'The output treemap shapes', 'viewShape',
            True, False, True)

        self.addColorPropertyParameter(
            'colors', 'The output treemap colors', 'viewColor',
            True, False, True)

        self.addColorPropertyParameter(
            'border colors', 'The output treemap border colors',
            'viewBorderColor', True, False, True)

        self.addDoublePropertyParameter(
            'border widths', 'The output treemap border widths',
            'viewBorderWidth', True, False, True)

    def check(self):
        # Input graph must be a tree for the treemap layout
        if not tlp.TreeTest.isTree(self.graph):
            return (False, 'The graph must be a tree.')

        return (True, '')

    def run(self):
        # retrieve parameter values
        metric = self.dataSet['metric']
        aspectRatio = self.dataSet['aspect ratio']
        treeMapType = self.dataSet['treemap type']
        borderColor = self.dataSet['border color']
        layout = self.dataSet['layout']
        sizes = self.dataSet['sizes']
        shapes = self.dataSet['shapes']
        colors = self.dataSet['colors']
        borderColors = self.dataSet['border colors']
        borderWidths = self.dataSet['border widths']

        # call the 'Squarified Tree Map' layout algorithm
        params = tlp.getDefaultPluginParameters('Squarified Tree Map',
                                                self.graph)
        params['Node Size'] = sizes
        params['Node Shape'] = shapes
        params['metric'] = metric
        params['Aspect Ratio'] = aspectRatio
        params['Treemap Type'] = treeMapType

        self.graph.applyLayoutAlgorithm('Squarified Tree Map', layout, params)

        # set edge colors and border colors to be fully transparent
        colors.setAllEdgeValue(tlp.Color(0, 0, 0, 0))
        borderColors.setAllEdgeValue(tlp.Color(0, 0, 0, 0))

        # set new border color for nodes
        borderColors.setAllNodeValue(borderColor)
        # ensure node borders are visible
        borderWidths.setAllNodeValue(1.0)

        # set the square shape to all leaf nodes
        for n in self.graph.getNodes():
            if self.graph.outdeg(n) == 0:
                shapes[n] = tlp.NodeShape.Square

        return True


pluginDoc = """
Enables to easily configure a treemap layout visualisation for a tree.
As the treemap layout is different from classical node link diagram
representation, some visual properties setup has to be done in order
to get an aesthetic visualization of it in Tulip.
This plugin takes care of calling the 'Squarified Tree Map' layout algorithm
and adjust some visual properties to get a correct rendering of the treemap.
"""

# The line below does the magic to register the plugin into the plugin database
# and updates the GUI to make it accessible through the menus.
tulipplugins.registerPlugin(
    'SquarifiedTreeMapHelper', 'Squarified Tree Map Helper',
    'Antoine Lambert', '27/04/2017', pluginDoc, '1.0')



from tulip import tlp
import tulipplugins

class Test(tlp.Algorithm):
	def init(self, context):
		tlp.Algorithm.init(self, context)
		self.addStringParameter("property", "the property to set", "", True, True, True)
	def check(self):
		return (True, "")
	def run(self):
		return True

tulipplugins.registerPlugin("Test", "A simple Python plugin test", "", "19/07/2016", "", "1.0")



# This file is part of Tulip (http://tulip.labri.fr)
#
# Authors: David Auber and the Tulip development Team
# from LaBRI, University of Bordeaux
#
# Tulip is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3
# of the License, or (at your option) any later version.
#
# Tulip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.

# Tulip Python plugin for mapping metric values to alpha component
# of graph element colors. In other words, it enables to compute
# the graph elements transparency according to the values
# stored in a numeric property of a graph.

from tulip import tlp
import tulipplugins
import math


def clamp(n, smallest, largest):
    return max(smallest, min(n, largest))


def getFactor(v, r):
    if r == 0:
        r = 1
    if v < 0:
        v = 0
    if v > r:
        v = r
    return v / r


mappingTypeDoc = """
That parameter defines the type of alpha mapping to perform.
For the linear case, the minimum value of the input numeric property
is mapped to a minimum alpha value picked by the user, the maximum
value is mapped to a maximum alpha value picked by the user, and a
linear interpolation is used between both to compute the associated
alpha of the graph element color.<br/>
For the logarithmic case, input numeric properties values are first
mapped in the [1, +inf[ range. Then the log of each mapped value is
computed and used to compute the associated alpha value of the graph
element color trough a linear interpolation between 0 and the log of
the mapped maximum value of graph elements.<br/>
If uniform, this is the same except for the interpolation: the values
are sorted, numbered, and a linear interpolation is used on those numbers
(in other words, only the order is taken into account, not the actual values).
"""


class AlphaMapping(tlp.ColorAlgorithm):

    def __init__(self, context):
        tlp.ColorAlgorithm.__init__(self, context)

        self.addNumericPropertyParameter(
            'input property',
            'The input numeric property from which to compute alpha mapping',
            'viewMetric')

        self.addStringCollectionParameter(
            'target',
            'Whether alpha values are computed for nodes or edges',
            'nodes;edges')

        self.addStringCollectionParameter(
            'type', mappingTypeDoc, 'linear;logarithmic;uniform')

        self.addUnsignedIntegerParameter(
            'min alpha',
            ('The minimum alpha value (between 0 and 255) '
             'to map on graph elements colors'), '0')

        self.addUnsignedIntegerParameter(
            'max alpha',
            ('The maximum alpha value (between 0 and 255) '
             'to map on graph elements colors'), '255')

    def run(self):
        vColor = self.graph.getColorProperty("viewColor")
        inputMetric = self.dataSet['input property']
        minAlpha = clamp(self.dataSet['min alpha'], 0, 255)
        maxAlpha = clamp(self.dataSet['max alpha'], 0, 255)
        target = self.dataSet['target']
        mappingType = self.dataSet['type']

        if minAlpha > maxAlpha:
            minAlpha, maxAlpha = maxAlpha, minAlpha

        if mappingType == 'uniform':
            inputMetric = inputMetric.copyProperty(self.graph)
            inputMetric.uniformQuantification(300)

        elif mappingType == 'logarithmic':
            inputMetric = inputMetric.copyProperty(self.graph)
            minN = inputMetric.getNodeDoubleMin()
            minE = inputMetric.getEdgeDoubleMin()
            for n in self.graph.getNodes():
                inputMetric[n] = math.log(1+inputMetric[n]-minN)
            for e in self.graph.getEdges():
                inputMetric[e] = math.log(1+inputMetric[e]-minE)

        if target == 'nodes':
            minValue = inputMetric.getNodeDoubleMin(self.graph)
            maxValue = inputMetric.getNodeDoubleMax(self.graph)

            for n in self.graph.nodes():
                val = inputMetric[n]
                t = getFactor(val - minValue, maxValue - minValue)
                alpha = minAlpha + t * (maxAlpha - minAlpha)
                color = vColor[n]
                color[3] = int(alpha)
                self.result[n] = color

        else:
            minValue = inputMetric.getEdgeDoubleMin(self.graph)
            maxValue = inputMetric.getEdgeDoubleMax(self.graph)

            for e in self.graph.edges():
                val = inputMetric[e]
                t = getFactor(val - minValue, maxValue - minValue)
                alpha = minAlpha + t * (maxAlpha - minAlpha)
                color = vColor[e]
                color[3] = int(alpha)
                self.result[e] = color

        return True


pluginDoc = """
Map metric values to alpha component of graph element colors.
In other words, it enables to compute the graph elements transparency
according to the values stored in a numeric property of a graph.
"""

# The line below does the magic to register the plugin into the plugin database
# and updates the GUI to make it accessible through the menus.
tulipplugins.registerPlugin('AlphaMapping', 'Alpha Mapping', 'Antoine Lambert',
                            '20/04/2017', pluginDoc, '1.1')



# This file is part of Tulip (http://tulip.labri.fr)
#
# Authors: David Auber and the Tulip development Team
# from LaBRI, University of Bordeaux
#
# Tulip is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3
# of the License, or (at your option) any later version.
#
# Tulip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.

import unittest
import sys

from tulip import tlp

import tulipplugins

plugin_name = 'Test Algorithm'
plugin_author = 'author'
plugin_date = 'date'
plugin_info = 'info'
plugin_version = 'version'
plugin_group = 'group'

boolean_param_name = 'bool'
boolean_param_default_value = True
boolean_param_value = False

int_param_name = 'int'
int_param_default_value = 5
int_param_value = -10

float_param_name = 'float'
float_param_default_value = 53.35
float_param_value = 156.876

string_param_name = 'string'
string_param_default_value = 'foo'
string_param_value = 'bar'

boolean_prop_param_name = 'bool_prop'
boolean_prop_param_default_value = 'viewSelection'
boolean_prop_param_value = 'prop_bool'

color_prop_param_name = 'color_prop'
color_prop_param_default_value = 'viewColor'
color_prop_param_value = 'prop_color'

double_prop_param_name = 'double_prop'
double_prop_param_default_value = 'viewMetric'
double_prop_param_value = 'color_prop'

int_prop_param_name = 'int_prop'
int_prop_param_default_value = 'viewShape'
int_prop_param_value = 'prop_int'

layout_prop_param_name = 'layout_prop'
layout_prop_param_default_value = 'viewLayout'
layout_prop_param_value = 'prop_layout'

size_prop_param_name = 'size_prop'
size_prop_param_default_value = 'viewSize'
size_prop_param_value = 'prop_size'

string_prop_param_name = 'string_prop'
string_prop_param_default_value = 'viewLabel'
string_prop_param_value = 'prop_string'

boolean_vec_prop_param_name = 'bool_vec_prop'
boolean_vec_prop_param_default_value = 'bool_vector'

color_vec_prop_param_name = 'color_vec_prop'
color_vec_prop_param_default_value = 'color_vector'

double_vec_prop_param_name = 'double_vec_prop'
double_vec_prop_param_default_value = 'double_vector'

int_vec_prop_param_name = 'int_vec_prop'
int_vec_prop_param_default_value = 'int_vector'

coord_vec_prop_param_name = 'coord_vec_prop'
coord_vec_prop_param_default_value = 'coord_vector'

size_vec_prop_param_name = 'size_vec_prop'
size_vec_prop_param_default_value = 'size_vector'

string_vec_prop_param_name = 'string_vec_prop'
string_vec_prop_param_default_value = 'string_vector'

color_param_name = 'color'
color_param_default_value = '(128,128,128,128)'
color_param_value = tlp.Color.Blue
color_param_tuple_value = (0, 0, 255)

color_scale_param_name = 'color_scale'
color_scale_param_default_value = '((255,0,0,255),(0,255,0,255),(0,0,255,255))'
color_scale_default_value_dict = {0.0: (255, 0, 0, 255),
                                  0.5: (0, 255, 0, 255),
                                  1.0: (0, 0, 255, 255)}
color_scale_param_value_dict = {0.0: (255, 0, 255, 255),
                                0.5: (128, 255, 0, 255),
                                1.0: (128, 0, 255, 255)}
color_scale_param_value_list = [(255, 0, 255, 255), (128, 255, 0, 255),
                                (128, 0, 255, 255)]
color_scale_param_value = tlp.ColorScale(color_scale_param_value_dict)

dir_param_name = 'dir'
dir_param_default_value = '/tmp'
dir_param_value = '/home/toto'

file_param_name = 'file'
file_param_default_value = '/tmp/foo.csv'
file_param_value = '/home/toto/bar.csv'

string_collection_param_name = 'string collection'
string_collection_param_default_value = 'foo;bar'
string_collection_param_value = 'bar'

out_boolean_param_name = 'bool_out'
out_boolean_param_value = True
out_int_param_name = 'int_out'
out_int_param_value = 2
out_float_param_name = 'float_out'
out_float_param_value = 0.5
out_string_param_name = 'string_out'
out_string_param_value = 'toto'

parameters_value = {}


class TestAlgorithm(tlp.Algorithm):

    def __init__(self, context):
        tlp.Algorithm.__init__(self, context)

        self.addBooleanParameter(
            boolean_param_name,
            defaultValue=str(boolean_param_default_value))

        self.addColorParameter(
            color_param_name,
            defaultValue=color_param_default_value)

        self.addColorScaleParameter(
            color_scale_param_name,
            defaultValue=color_scale_param_default_value)

        self.addIntegerParameter(
            int_param_name,
            defaultValue=str(int_param_default_value))

        self.addFloatParameter(
            float_param_name,
            defaultValue=str(float_param_default_value))

        self.addStringParameter(
            string_param_name,
            defaultValue=str(string_param_default_value))

        self.addDirectoryParameter(
            dir_param_name,
            defaultValue=dir_param_default_value)

        self.addFileParameter(
            file_param_name,
            defaultValue=file_param_default_value)

        self.addStringCollectionParameter(
            string_collection_param_name,
            defaultValue=string_collection_param_default_value)

        self.addBooleanPropertyParameter(
            boolean_prop_param_name,
            defaultValue=boolean_prop_param_default_value)

        self.addColorPropertyParameter(
            color_prop_param_name,
            defaultValue=color_prop_param_default_value)

        self.addDoublePropertyParameter(
            double_prop_param_name,
            defaultValue=double_prop_param_default_value)

        self.addIntegerPropertyParameter(
            int_prop_param_name,
            defaultValue=int_prop_param_default_value)

        self.addLayoutPropertyParameter(
            layout_prop_param_name,
            defaultValue=layout_prop_param_default_value)

        self.addSizePropertyParameter(
            size_prop_param_name,
            defaultValue=size_prop_param_default_value)

        self.addStringPropertyParameter(
            string_prop_param_name,
            defaultValue=string_prop_param_default_value)

        self.addBooleanVectorPropertyParameter(
            boolean_vec_prop_param_name,
            defaultValue=boolean_vec_prop_param_default_value)

        self.addColorVectorPropertyParameter(
            color_vec_prop_param_name,
            defaultValue=color_vec_prop_param_default_value)

        self.addDoubleVectorPropertyParameter(
            double_vec_prop_param_name,
            defaultValue=double_vec_prop_param_default_value)

        self.addIntegerVectorPropertyParameter(
            int_vec_prop_param_name,
            defaultValue=int_vec_prop_param_default_value)

        self.addCoordVectorPropertyParameter(
            coord_vec_prop_param_name,
            defaultValue=coord_vec_prop_param_default_value)

        self.addSizeVectorPropertyParameter(
            size_vec_prop_param_name,
            defaultValue=size_vec_prop_param_default_value)

        self.addStringVectorPropertyParameter(
            string_vec_prop_param_name,
            defaultValue=string_vec_prop_param_default_value)

        self.addBooleanParameter(out_boolean_param_name, outParam=True)
        self.addIntegerParameter(out_int_param_name, outParam=True)
        self.addFloatParameter(out_float_param_name, outParam=True)
        self.addStringParameter(out_string_param_name, outParam=True)

    def check(self):
        return (True, 'Ok')

    def run(self):
        p = parameters_value
        d = self.dataSet
        p[boolean_param_name] = d[boolean_param_name]
        p[int_param_name] = d[int_param_name]
        p[float_param_name] = d[float_param_name]
        p[string_param_name] = d[string_param_name]
        p[string_collection_param_name] = d[string_collection_param_name]

        p[color_param_name] = tlp.Color(d[color_param_name])
        p[color_scale_param_name] = tlp.ColorScale(d[color_scale_param_name])

        p[boolean_prop_param_name] = d[boolean_prop_param_name]
        p[color_prop_param_name] = d[color_prop_param_name]
        p[double_prop_param_name] = d[double_prop_param_name]
        p[int_prop_param_name] = d[int_prop_param_name]
        p[layout_prop_param_name] = d[layout_prop_param_name]
        p[size_prop_param_name] = d[size_prop_param_name]
        p[string_prop_param_name] = d[string_prop_param_name]
        p[boolean_vec_prop_param_name] = d[boolean_vec_prop_param_name]
        p[color_vec_prop_param_name] = d[color_vec_prop_param_name]
        p[double_vec_prop_param_name] = d[double_vec_prop_param_name]
        p[int_vec_prop_param_name] = d[int_vec_prop_param_name]
        p[coord_vec_prop_param_name] = d[coord_vec_prop_param_name]
        p[size_vec_prop_param_name] = d[size_vec_prop_param_name]
        p[string_vec_prop_param_name] = d[string_vec_prop_param_name]

        d[out_boolean_param_name] = out_boolean_param_value
        d[out_int_param_name] = out_int_param_value
        d[out_float_param_name] = out_float_param_value
        d[out_string_param_name] = out_string_param_value

        return True


tulipplugins.registerPluginOfGroup(
    'TestAlgorithm', plugin_name, plugin_author,
    plugin_date, plugin_info, plugin_version, plugin_group)


class TestPluginParameters(unittest.TestCase):

    def setUp(self):
        self.graph = tlp.newGraph()

        self.boolean_prop = self.graph.getBooleanProperty(
            boolean_prop_param_value)

        self.color_prop = self.graph.getColorProperty(
            color_prop_param_value)

        self.double_prop = self.graph.getDoubleProperty(
            double_prop_param_value)

        self.int_prop = self.graph.getIntegerProperty(
            int_prop_param_value)

        self.layout_prop = self.graph.getLayoutProperty(
            layout_prop_param_value)

        self.size_prop = self.graph.getSizeProperty(size_prop_param_value)

        self.string_prop = self.graph.getStringProperty(
            string_prop_param_value)

        self.boolean_vec_prop = self.graph.getBooleanVectorProperty(
            boolean_vec_prop_param_default_value)

        self.color_vec_prop = self.graph.getColorVectorProperty(
            color_vec_prop_param_default_value)

        self.double_vec_prop = self.graph.getDoubleVectorProperty(
            double_vec_prop_param_default_value)

        self.int_vec_prop = self.graph.getIntegerVectorProperty(
            int_vec_prop_param_default_value)

        self.coord_vec_prop = self.graph.getCoordVectorProperty(
            coord_vec_prop_param_default_value)

        self.size_vec_prop = self.graph.getSizeVectorProperty(
            size_vec_prop_param_default_value)

        self.string_vec_prop = self.graph.getStringVectorProperty(
            string_vec_prop_param_default_value)

    def tearDown(self):
        self.graph = None

    def test_plugin_is_registered(self):
        self.assertIn(plugin_name, tlp.getAlgorithmPluginsList())
        plugin = tlp.PluginLister.pluginInformation(plugin_name)
        self.assertEqual(plugin.category(), 'Algorithm')
        self.assertEqual(plugin.name(), plugin_name)
        self.assertEqual(plugin.author(), plugin_author)
        self.assertEqual(plugin.date(), plugin_date)
        self.assertEqual(plugin.info(), plugin_info)
        self.assertEqual(plugin.release(), plugin_version)
        self.assertEqual(plugin.group(), plugin_group)
        self.assertEqual(plugin.programmingLanguage(), 'Python')

    def test_plugin_default_parameters(self):
        plugin_default_params = tlp.getDefaultPluginParameters(plugin_name,
                                                               self.graph)

        self.assertIn(boolean_param_name, plugin_default_params)
        self.assertEqual(type(plugin_default_params[boolean_param_name]), bool)
        self.assertEqual(plugin_default_params[boolean_param_name],
                         boolean_param_default_value)

        self.assertIn(color_param_name, plugin_default_params)

        self.assertTrue(isinstance(plugin_default_params[color_param_name],
                        tlp.Color))

        self.assertEqual(repr(plugin_default_params[color_param_name]),
                         color_param_default_value)

        self.assertIn(color_scale_param_name,
                      plugin_default_params)

        self.assertTrue(
            isinstance(plugin_default_params[color_scale_param_name],
                       tlp.ColorScale))

        self.assertEqual(plugin_default_params[color_scale_param_name],
                         tlp.ColorScale(color_scale_default_value_dict))

        self.assertIn(int_param_name, plugin_default_params)
        if sys.version_info >= (3,):
            self.assertEqual(type(plugin_default_params[int_param_name]), int)
        else:
            self.assertEqual(type(plugin_default_params[int_param_name]), long) # noqa
        self.assertEqual(plugin_default_params[int_param_name],
                         int_param_default_value)

        self.assertIn(float_param_name, plugin_default_params)
        self.assertEqual(type(plugin_default_params[float_param_name]), float)
        self.assertEqual(plugin_default_params[float_param_name],
                         float_param_default_value)

        self.assertIn(string_param_name, plugin_default_params)
        self.assertEqual(type(plugin_default_params[string_param_name]), str)
        self.assertEqual(plugin_default_params[string_param_name],
                         string_param_default_value)

        self.assertIn(string_collection_param_name, plugin_default_params)
        self.assertEqual(
            type(plugin_default_params[string_collection_param_name]), str)
        self.assertEqual(plugin_default_params[string_collection_param_name],
                         string_collection_param_default_value.split(';')[0])

        self.assertIn(boolean_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[boolean_prop_param_name],
                       tlp.BooleanProperty))
        self.assertEqual(
            plugin_default_params[boolean_prop_param_name].getName(),
            boolean_prop_param_default_value)

        self.assertIn(color_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[color_prop_param_name],
                       tlp.ColorProperty))
        self.assertEqual(
            plugin_default_params[color_prop_param_name].getName(),
            color_prop_param_default_value)

        self.assertIn(double_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[double_prop_param_name],
                       tlp.DoubleProperty))
        self.assertEqual(
            plugin_default_params[double_prop_param_name].getName(),
            double_prop_param_default_value)

        self.assertIn(int_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[int_prop_param_name],
                       tlp.IntegerProperty))
        self.assertEqual(
            plugin_default_params[int_prop_param_name].getName(),
            int_prop_param_default_value)

        self.assertIn(layout_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[layout_prop_param_name],
                       tlp.LayoutProperty))
        self.assertEqual(
            plugin_default_params[layout_prop_param_name].getName(),
            layout_prop_param_default_value)

        self.assertIn(size_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[size_prop_param_name],
                       tlp.SizeProperty))
        self.assertEqual(
            plugin_default_params[size_prop_param_name].getName(),
            size_prop_param_default_value)

        self.assertIn(string_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[string_prop_param_name],
                       tlp.StringProperty))
        self.assertEqual(
            plugin_default_params[string_prop_param_name].getName(),
            string_prop_param_default_value)

        self.assertIn(boolean_vec_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[boolean_vec_prop_param_name],
                       tlp.BooleanVectorProperty))
        self.assertEqual(
            plugin_default_params[boolean_vec_prop_param_name].getName(),
            boolean_vec_prop_param_default_value)

        self.assertIn(color_vec_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[color_vec_prop_param_name],
                       tlp.ColorVectorProperty))
        self.assertEqual(
            plugin_default_params[color_vec_prop_param_name].getName(),
            color_vec_prop_param_default_value)

        self.assertIn(double_vec_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[double_vec_prop_param_name],
                       tlp.DoubleVectorProperty))
        self.assertEqual(
            plugin_default_params[double_vec_prop_param_name].getName(),
            double_vec_prop_param_default_value)

        self.assertIn(int_vec_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[int_vec_prop_param_name],
                       tlp.IntegerVectorProperty))
        self.assertEqual(
            plugin_default_params[int_vec_prop_param_name].getName(),
            int_vec_prop_param_default_value)

        self.assertIn(coord_vec_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[coord_vec_prop_param_name],
                       tlp.CoordVectorProperty))
        self.assertEqual(
            plugin_default_params[coord_vec_prop_param_name].getName(),
            coord_vec_prop_param_default_value)

        self.assertIn(size_vec_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[size_vec_prop_param_name],
                       tlp.SizeVectorProperty))
        self.assertEqual(
            plugin_default_params[size_vec_prop_param_name].getName(),
            size_vec_prop_param_default_value)

        self.assertIn(string_vec_prop_param_name, plugin_default_params)
        self.assertTrue(
            isinstance(plugin_default_params[string_vec_prop_param_name],
                       tlp.StringVectorProperty))
        self.assertEqual(
            plugin_default_params[string_vec_prop_param_name].getName(),
            string_vec_prop_param_default_value)

    def test_plugin_in_parameters(self):
        plugin_params = tlp.getDefaultPluginParameters(plugin_name, self.graph)

        plugin_params[boolean_param_name] = boolean_param_value
        plugin_params[int_param_name] = int_param_value
        plugin_params[float_param_name] = float_param_value
        plugin_params[string_param_name] = string_param_value
        plugin_params[
            string_collection_param_name] = string_collection_param_value
        plugin_params[color_scale_param_name] = color_scale_param_value
        plugin_params[color_param_name] = color_param_value

        plugin_params[boolean_prop_param_name] = self.boolean_prop
        plugin_params[color_prop_param_name] = self.color_prop
        plugin_params[double_prop_param_name] = self.double_prop
        plugin_params[int_prop_param_name] = self.int_prop
        plugin_params[layout_prop_param_name] = self.layout_prop
        plugin_params[size_prop_param_name] = self.size_prop
        plugin_params[string_prop_param_name] = self.string_prop
        plugin_params[boolean_vec_prop_param_name] = self.boolean_vec_prop
        plugin_params[color_vec_prop_param_name] = self.color_vec_prop
        plugin_params[double_vec_prop_param_name] = self.double_vec_prop
        plugin_params[int_vec_prop_param_name] = self.int_vec_prop
        plugin_params[coord_vec_prop_param_name] = self.coord_vec_prop
        plugin_params[size_vec_prop_param_name] = self.size_vec_prop
        plugin_params[string_vec_prop_param_name] = self.string_vec_prop

        plugin_ret = self.graph.applyAlgorithm(plugin_name, plugin_params)
        self.assertTrue(plugin_ret[0])

        self.assertEqual(parameters_value[boolean_param_name],
                         boolean_param_value)
        self.assertEqual(parameters_value[int_param_name], int_param_value)
        self.assertEqual(parameters_value[float_param_name], float_param_value)
        self.assertEqual(parameters_value[string_param_name],
                         string_param_value)
        self.assertEqual(parameters_value[string_param_name],
                         string_param_value)
        self.assertEqual(parameters_value[string_collection_param_name],
                         string_collection_param_value)
        self.assertEqual(parameters_value[color_param_name], color_param_value)
        self.assertEqual(parameters_value[color_scale_param_name],
                         color_scale_param_value)

        self.assertEqual(parameters_value[boolean_prop_param_name],
                         self.boolean_prop)
        self.assertEqual(parameters_value[color_prop_param_name],
                         self.color_prop)
        self.assertEqual(parameters_value[double_prop_param_name],
                         self.double_prop)
        self.assertEqual(parameters_value[int_prop_param_name], self.int_prop)
        self.assertEqual(parameters_value[layout_prop_param_name],
                         self.layout_prop)
        self.assertEqual(parameters_value[size_prop_param_name],
                         self.size_prop)
        self.assertEqual(parameters_value[string_prop_param_name],
                         self.string_prop)
        self.assertEqual(parameters_value[boolean_vec_prop_param_name],
                         self.boolean_vec_prop)
        self.assertEqual(parameters_value[color_vec_prop_param_name],
                         self.color_vec_prop)
        self.assertEqual(parameters_value[double_vec_prop_param_name],
                         self.double_vec_prop)
        self.assertEqual(parameters_value[int_vec_prop_param_name],
                         self.int_vec_prop)
        self.assertEqual(parameters_value[coord_vec_prop_param_name],
                         self.coord_vec_prop)
        self.assertEqual(parameters_value[size_vec_prop_param_name],
                         self.size_vec_prop)
        self.assertEqual(parameters_value[string_vec_prop_param_name],
                         self.string_vec_prop)

        plugin_params[color_param_name] = color_param_tuple_value
        del parameters_value[color_param_name]

        plugin_params[color_scale_param_name] = color_scale_param_value_dict
        del parameters_value[color_scale_param_name]

        plugin_ret = self.graph.applyAlgorithm(plugin_name, plugin_params)
        self.assertTrue(plugin_ret[0])

        self.assertEqual(parameters_value[color_param_name], color_param_value)
        self.assertEqual(parameters_value[color_scale_param_name],
                         color_scale_param_value)

        plugin_params[color_scale_param_name] = color_scale_param_value_list
        del parameters_value[color_scale_param_name]

        plugin_ret = self.graph.applyAlgorithm(plugin_name, plugin_params)
        self.assertTrue(plugin_ret[0])

        self.assertEqual(parameters_value[color_scale_param_name],
                         color_scale_param_value)

        plugin_params[string_collection_param_name] = 'toto'

        with self.assertRaises(Exception) as cm:
            plugin_ret = self.graph.applyAlgorithm(plugin_name, plugin_params)
            self.assertContains(
                cm.exception.args[0], 'Possible values are : foo, bar')

    def test_plugin_out_parameters(self):
        plugin_params = tlp.getDefaultPluginParameters(plugin_name, self.graph)

        plugin_ret = self.graph.applyAlgorithm(plugin_name, plugin_params)
        self.assertTrue(plugin_ret[0])

        self.assertIn(out_boolean_param_name, plugin_params)
        self.assertEqual(plugin_params[out_boolean_param_name],
                         out_boolean_param_value)

        self.assertIn(out_int_param_name, plugin_params)
        self.assertEqual(plugin_params[out_int_param_name],
                         out_int_param_value)

        self.assertIn(out_float_param_name, plugin_params)
        self.assertEqual(plugin_params[out_float_param_name],
                         out_float_param_value)

        self.assertIn(out_string_param_name, plugin_params)
        self.assertEqual(plugin_params[out_string_param_name],
                         out_string_param_value)



# automatically generates the file tulippluginsdocumentation.rst
# by dynamically introspecting the Tulip plugins metadata

from __future__ import print_function
import sys
if sys.version_info[0] == 2:
    reload(sys) # noqa
    sys.setdefaultencoding('utf8')
from tulip import tlp # noqa
import tulipgui # noqa
import tabulate # noqa
import re # noqa
import os # noqa


def safeprint(s, file=None):
    try:
        print(s, file=file)
    except UnicodeEncodeError:
        print(s.encode('utf8'), file=file)


def utf8len(s):
    if sys.version_info >= (3,):
        return len(s)
    else:
        return len(s.encode('utf8'))


tulip_build_dir = os.environ['TULIP_BUILD_DIR']
tlp.loadTulipPluginsFromDir('%s/plugins/clustering' % tulip_build_dir)
tlp.loadTulipPluginsFromDir('%s/plugins/colors' % tulip_build_dir)
tlp.loadTulipPluginsFromDir('%s/plugins/export' % tulip_build_dir)
tlp.loadTulipPluginsFromDir('%s/plugins/general' % tulip_build_dir)
tlp.loadTulipPluginsFromDir('%s/plugins/import' % tulip_build_dir)
tlp.loadTulipPluginsFromDir('%s/plugins/layout' % tulip_build_dir)
tlp.loadTulipPluginsFromDir('%s/plugins/metric' % tulip_build_dir)
tlp.loadTulipPluginsFromDir('%s/plugins/selection' % tulip_build_dir)
tlp.loadTulipPluginsFromDir('%s/plugins/sizes' % tulip_build_dir)
tlp.loadTulipPluginsFromDir('%s/plugins/string' % tulip_build_dir)
tlp.loadTulipPluginsFromDir('%s/plugins/test' % tulip_build_dir)
tlp.loadTulipPluginsFromDir(os.environ['TULIP_PYTHON_PLUGINS_DIR'])
tlp.loadTulipPluginsFromDir(os.environ['TULIPGUI_PYTHON_PLUGINS_DIR'])

if sys.version_info >= (3,):
    f = open('tulippluginsdocumentation.rst', 'w', encoding='utf-8')
else:
    f = open('tulippluginsdocumentation.rst', 'w')

safeprint("""
.. |br| raw:: html

   <br />

.. |bstart| raw:: html

   <b>

.. |bend| raw:: html

   </b>

.. |istart| raw:: html

   <i>

.. |iend| raw:: html

   </i>

.. |listart| raw:: html

   <li>

.. |liend| raw:: html

   </li>

.. |ulstart| raw:: html

   <ul>

.. |ulend| raw:: html

   </ul>

.. |pstart| raw:: html

   <p>

.. |pend| raw:: html

   </p>

""", file=f)

safeprint('.. py:currentmodule:: tulip\n', file=f)

safeprint('.. _tulippluginsdoc:\n', file=f)


def writeSection(title, sectionChar):
    safeprint(title, file=f)
    underline = ''
    for i in range(utf8len(title)):
        underline += sectionChar
    safeprint(underline+'\n', file=f)


def replaceHtmlTags(doc, htmlTag, htmlTagStartReplacement,
                    htmlTagEndReplacement):
    tmp = re.sub('^<%s>' % htmlTag,
                 '|%s| ' % htmlTagStartReplacement, doc)
    tmp = re.sub('^</%s>' % htmlTag,
                 '|%s| ' % htmlTagEndReplacement, tmp)
    tmp = re.sub('^<%s>' % htmlTag.upper(),
                 '|%s| ' % htmlTagStartReplacement, tmp)
    tmp = re.sub('^</%s>' % htmlTag.upper(),
                 '|%s| ' % htmlTagEndReplacement, tmp)
    tmp = tmp.replace('<%s>' % htmlTag,
                      ' |%s| ' % htmlTagStartReplacement)
    tmp = tmp.replace('</%s>' % htmlTag,
                      ' |%s| ' % htmlTagEndReplacement)
    tmp = tmp.replace('<%s>' % htmlTag.upper(),
                      ' |%s| ' % htmlTagStartReplacement)
    tmp = tmp.replace('</%s>' % htmlTag.upper(),
                      ' |%s| ' % htmlTagEndReplacement)
    return tmp


def formatSphinxDoc(doc):
    doc = re.sub('^\n', '', doc)
    doc = re.sub('\n$', '', doc)
    doc = replaceHtmlTags(doc, 'br', 'br', 'br')
    doc = replaceHtmlTags(doc, 'br/', 'br', 'br')
    doc = replaceHtmlTags(doc, 'b', 'bstart', 'bend')
    doc = replaceHtmlTags(doc, 'i', 'istart', 'iend')
    doc = replaceHtmlTags(doc, 'li', 'listart', 'liend')
    doc = replaceHtmlTags(doc, 'ul', 'ulstart', 'ulend')
    doc = replaceHtmlTags(doc, 'p', 'pstart', 'pend')
    return doc


def getTulipPythonType(tulipType):
    if tulipType == 'BooleanProperty':
        return ':class:`tlp.BooleanProperty`'
    elif tulipType == 'ColorProperty':
        return ':class:`tlp.ColorProperty`'
    elif tulipType == 'DoubleProperty':
        return ':class:`tlp.DoubleProperty`'
    elif tulipType == 'IntegerProperty':
        return ':class:`tlp.IntegerProperty`'
    elif tulipType == 'LayoutProperty':
        return ':class:`tlp.LayoutProperty`'
    elif tulipType == 'SizeProperty':
        return ':class:`tlp.SizeProperty`'
    elif tulipType == 'StringProperty':
        return ':class:`tlp.StringProperty`'
    elif tulipType == 'NumericProperty':
        return ':class:`tlp.NumericProperty`'
    elif tulipType == 'PropertyInterface':
        return ':class:`tlp.PropertyInterface`'
    elif tulipType == 'StringCollection':
        return ':class:`tlp.StringCollection`'
    elif tulipType == 'ColorScale':
        return ':class:`tlp.ColorScale`'
    elif tulipType == 'Color':
        return ':class:`tlp.Color`'
    elif tulipType == 'Boolean':
        return ':const:`bool`'
    elif tulipType.startswith('float'):
        return ':const:`float`'
    elif 'integer' in tulipType:
        return ':const:`int`'
    elif tulipType == 'string':
        return ':const:`str`'
    else:
        return tulipType


writeSection('Tulip plugins documentation', '=')

safeprint("""
In this section, you can find some documentation regarding the C++ algorithm
plugins bundled in the Tulip software but also with the Tulip Python modules
installable through the pip tool. In particular, an exhaustive description of
the input and output parameters for each plugin is given.
To learn how to call all these algorithms in Python, you can refer to the
:ref:`Applying an algorithm on a graph <applyGraphAlgorithm>` section.
The plugins documentation is ordered according to their type.

.. warning:: If you use the Tulip Python bindings trough the classical Python
             interpreter, some plugins (Color Mapping, Convolution Clustering,
             File System Directory, GEXF, SVG Export, Website) require the
             :mod:`tulipgui` module to be imported before they can be called
             as they use Qt under the hood.
""" + '\n', file=f)

pluginsNames = tlp.PluginLister.availablePlugins()
plugins = {}
for pluginName in pluginsNames:
    plugin = tlp.PluginLister.pluginInformation(pluginName)
    if not plugin.category() in plugins:
        plugins[plugin.category()] = []
    plugins[plugin.category()].append(plugin)
for cat in sorted(plugins.keys()):
    if cat in ('Perspective', 'Panel', 'Node shape',
               'Edge extremity', 'Interactor'):
        continue
    writeSection(cat, '-')
    if cat == 'Algorithm':
        safeprint('.. _algorithmpluginsdoc:\n', file=f)
        safeprint(('To call these plugins, you must use the '
                   ':meth:`tlp.Graph.applyAlgorithm` method. See also '
                   ':ref:`Calling a general algorithm on a graph '
                   '<callGeneralAlgorithm>` for more details.\n'), file=f)
    elif cat == 'Coloring':
        safeprint('.. _colorpluginsdoc:\n', file=f)
        safeprint(('To call these plugins, you must use the '
                   ':meth:`tlp.Graph.applyColorAlgorithm` method. See also '
                   ':ref:`Calling a property algorithm on a graph '
                   '<callPropertyAlgorithm>` for more details.\n'), file=f)
    elif cat == 'Export':
        safeprint('.. _exportpluginsdoc:\n', file=f)
        safeprint(('To call these plugins, you must use the '
                   ':func:`tlp.exportGraph` function.\n'), file=f)
    elif cat == 'Import':
        safeprint('.. _importpluginsdoc:\n', file=f)
        safeprint(('To call these plugins, you must use the '
                   ':func:`tlp.importGraph` function.\n'), file=f)
    elif cat == 'Labeling':
        safeprint('.. _stringpluginsdoc:\n', file=f)
        safeprint(('To call these plugins, you must use the '
                   ':meth:`tlp.Graph.applyStringAlgorithm` method. See also '
                   ':ref:`Calling a property algorithm on a graph '
                   '<callPropertyAlgorithm>` for more details.\n'), file=f)
    elif cat == 'Layout':
        safeprint('.. _layoutpluginsdoc:\n', file=f)
        safeprint(('To call these plugins, you must use the '
                   ':meth:`tlp.Graph.applyLayoutAlgorithm` method. See also '
                   ':ref:`Calling a property algorithm on a graph '
                   '<callPropertyAlgorithm>` for more details.\n'), file=f)
    elif cat == 'Measure':
        safeprint('.. _doublepluginsdoc:\n', file=f)
        safeprint(('To call these plugins, you must use the '
                   ':meth:`tlp.Graph.applyDoubleAlgorithm` method. See also '
                   ':ref:`Calling a property algorithm on a graph '
                   '<callPropertyAlgorithm>` for more details.\n'), file=f)
    elif cat == 'Resizing':
        safeprint('.. _sizepluginsdoc:\n', file=f)
        safeprint(('To call these plugins, you must use the '
                   ':meth:`tlp.Graph.applySizeAlgorithm` method. See also '
                   ':ref:`Calling a property algorithm on a graph '
                   '<callPropertyAlgorithm>` for more details.\n'), file=f)
    elif cat == 'Selection':
        safeprint('.. _booleanpluginsdoc:\n', file=f)
        safeprint(('To call these plugins, you must use the '
                   ':meth:`tlp.Graph.applyBooleanAlgorithm` method. See also '
                   ':ref:`Calling a property algorithm on a graph '
                   '<callPropertyAlgorithm>` for more details.\n'), file=f)

    for p in sorted(plugins[cat], key=lambda p: p.name()):
        writeSection(p.name(), '^')
        writeSection('Description', '"')
        infos = formatSphinxDoc(p.info())
        safeprint(infos+'\n', file=f)

        params = tlp.PluginLister.getPluginParameters(p.name())
        headers = ["name", "type", "default", "direction", "description"]
        paramsTable = []
        nbInParams = 0
        for param in params.getParameters():
            paramHelpHtml = param.getHelp()
            pattern = '<p class="help">'
            pos = paramHelpHtml.find(pattern)
            paramHelp = ''
            if pos != -1:
                pos2 = paramHelpHtml.rfind('</p>')
                paramHelp = paramHelpHtml[pos+len(pattern):pos2]
                paramHelp = formatSphinxDoc(paramHelp)
            pattern = '<b>type</b><td class="b">'
            pos = paramHelpHtml.find(pattern)
            paramType = ''
            if pos != -1:
                pos2 = paramHelpHtml.find('</td>', pos)
                paramType = paramHelpHtml[pos+len(pattern):pos2].replace(
                    ' (double precision)', '')
            if param.getName() == 'result' and 'Property' in paramType:
                continue
            paramDir = 'input / output'
            if param.getDirection() == tlp.IN_PARAM:
                paramDir = 'input'
                nbInParams = nbInParams+1
            elif param.getDirection() == tlp.OUT_PARAM:
                paramDir = 'output'
            else:
                nbInParams = nbInParams+1
            pattern = '<b>values</b><td class="b">'
            pos = paramHelpHtml.find(pattern)
            paramValues = ''
            if pos != -1:
                pos2 = paramHelpHtml.find('</td>', pos)
                paramValues = paramHelpHtml[pos+len(pattern):pos2]
                paramValues = formatSphinxDoc(paramValues)
            paramDefValue = param.getDefaultValue()
            if paramType == 'StringCollection':
                paramDefValue = paramDefValue[0:paramDefValue.find(';')]
            else:
                paramDefValue = paramDefValue.replace(
                    'false', ':const:`False`')
                paramDefValue = paramDefValue.replace('true', ':const:`True`')
            nonBreakingSpace = u'\xa0'
            if len(paramValues) > 0:
                paramDefValue += (u' |br| |br| |bstart| Values: |bend| |br| %s'
                                  % paramValues)
            valuesDoc = False

            paramName = param.getName().replace(
                'file::', '').replace('dir::', '')
            paramName = paramName.replace(' ', nonBreakingSpace)
            paramType = getTulipPythonType(
                paramType.replace(' ', nonBreakingSpace))
            paramDir = paramDir.replace(' ', nonBreakingSpace)
            if sys.version_info[0] == 2:
                paramName = paramName.decode('utf-8')
                paramType = paramType.decode('utf-8')
                paramDefValue = paramDefValue.decode('utf-8')
                paramDir = paramDir.decode('utf-8')
                paramHelp = paramHelp.decode('utf-8').replace('\n', ' |br| ')
            paramsTable.append([paramName, paramType, paramDefValue, paramDir,
                                paramHelp])
        if len(paramsTable) > 0:
            writeSection('Parameters', '"')
            safeprint(tabulate.tabulate(paramsTable, headers,
                                        tablefmt='grid') + '\n', file=f)
        writeSection('Calling the plugin from Python', '"')
        safeprint(('To call that plugin from Python, use the following '
                   'code snippet::\n'), file=f)
        safeprint(('  # get a dictionary filled with the default plugin '
                   'parameters values'), file=f)
        if p.category() != 'Import':
            safeprint(('  # graph is an instance of the tlp.Graph class\n'
                       '  params = tlp.getDefaultPluginParameters(\'%s\', '
                       'graph)\n') % p.name(), file=f)
        else:
            safeprint('  params = tlp.getDefaultPluginParameters(\'%s\')\n'
                      % p.name(), file=f)
        if nbInParams > 0 and not(len(paramsTable) == 1 and
                                  paramsTable[0][0] == 'result'):
            safeprint('  # set any input parameter value if needed', file=f)
            for paramData in paramsTable:
                safeprint('  # params[\'%s\'] = ...' % paramData[0], file=f)
            safeprint('', file=f)
        if p.category() == 'Import':
            safeprint('  graph = tlp.importGraph(\'%s\', params)' % p.name(),
                      file=f)
        elif p.category() == 'Export':
            safeprint('  outputFile = \'<path to a file>\'', file=f)
            safeprint(('  success = tlp.exportGraph(\'%s\', graph, outputFile,'
                       ' params)') % p.name(), file=f)
        elif p.category() == 'Layout':
            safeprint(('  # either create or get a layout property from the'
                       ' graph to store the result of the algorithm'), file=f)
            safeprint(('  resultLayout = graph.getLayoutProperty('
                       '\'resultLayout\')'), file=f)
            safeprint(('  success = graph.applyLayoutAlgorithm(\'%s\', '
                       'resultLayout, params)\n') % p.name(), file=f)
            safeprint(('  # or store the result of the algorithm in the'
                       ' default Tulip layout property named \'viewLayout\''),
                      file=f)
            safeprint(('  success = graph.applyLayoutAlgorithm(\'%s\', '
                       'params)\n') % p.name(), file=f)
        elif p.category() == 'Measure':
            safeprint(('  # either create or get a double property from the'
                       ' graph to store the result of the algorithm'), file=f)
            safeprint(('  resultMetric = graph.getDoubleProperty('
                       '\'resultMetric\')'), file=f)
            safeprint(('  success = graph.applyDoubleAlgorithm(\'%s\''
                       ', resultMetric, params)\n') % p.name(), file=f)
            safeprint(('  # or store the result of the algorithm in the'
                       ' default Tulip metric property named \'viewMetric\''),
                      file=f)
            safeprint(('  success = graph.applyDoubleAlgorithm(\'%s\','
                       ' params)\n') % p.name(), file=f)
        elif p.category() == 'Coloring':
            safeprint(('  # either create or get a color property from the'
                       ' graph to store the result of the algorithm'), file=f)
            safeprint(('  resultColor = graph.getColorProperty('
                       '\'resultColor\')'), file=f)
            safeprint(('  success = graph.applyColorAlgorithm(\'%s\','
                       ' resultColor, params)\n') % p.name(), file=f)
            safeprint(('  # or store the result of the algorithm in the'
                       ' default Tulip color property named \'viewColor\''),
                      file=f)
            safeprint(('  success = graph.applyColorAlgorithm(\'%s\', '
                       'params)\n') % p.name(), file=f)
        elif p.category() == 'Resizing':
            safeprint(('  # either create or get a size property from the'
                       ' graph to store the result of the algorithm'), file=f)
            safeprint('  resultSize = graph.getSizeProperty(\'resultSize\')',
                      file=f)
            safeprint(('  success = graph.applySizeAlgorithm(\'%s\','
                       ' resultSize, params)\n') % p.name(), file=f)
            safeprint(('  # or store the result of the algorithm in the'
                       ' default Tulip size property named \'viewSize\''),
                      file=f)
            safeprint(('  success = graph.applySizeAlgorithm(\'%s\', '
                       'params)\n') % p.name(), file=f)
        elif p.category() == 'Selection':
            safeprint(('  # either create or get a boolean property from the'
                       ' graph to store the result of the algorithm'), file=f)
            safeprint(('  resultSelection = graph.getBooleanProperty('
                       '\'resultSelection\')'), file=f)
            safeprint(('  success = graph.applyBooleanAlgorithm(\'%s\', '
                       'resultSelection, params)\n') % p.name(), file=f)
            safeprint(('  # or store the result of the algorithm in the '
                       'default Tulip boolean property named '
                       '\'viewSelection\''), file=f)
            safeprint(('  success = graph.applyBooleanAlgorithm(\'%s\', '
                       'params)\n') % p.name(), file=f)
        elif p.category() == 'Labeling':
            safeprint(('  # either create or get a string property from the '
                       'graph to store the result of the algorithm'), file=f)
            safeprint(('  resultString = graph.getStringProperty('
                       '\'resultString\')'), file=f)
            safeprint(('  success = graph.applyStringAlgorithm(\'%s\', '
                       'resultString, params)\n') % p.name(), file=f)
            safeprint(('  # or store the result of the algorithm in the '
                       'default Tulip string property named \'viewLabel\''),
                      file=f)
            safeprint(('  success = graph.applyStringAlgorithm(\'%s\', '
                       'params)\n') % p.name(), file=f)
        elif p.category() == 'Algorithm':
            safeprint(('  success = graph.applyAlgorithm(\'%s\', '
                       'params)\n') % p.name(), file=f)
        safeprint(('  # if the plugin declare any output parameter, its value'
                   ' can now be retrieved in the \'params\' dictionary\n'),
                  file=f)
f.close()



#!/usr/bin/env python
# Make Makefile
#
# October 2012
# Markus Chimani, markus.chimani@cs.tu-dortmund.de
# Carsten Gutwenger, carsten.gutwenger@cs.tu-dortmund.de
#########################################################

# use Python 3 print function on both Python 2.x and Python 3.x
from __future__ import print_function

import os, sys, fnmatch, posixpath, shutil, subprocess

# Simple hack for using "configparser" on both Python 2 and 3
if sys.hexversion > 0x03000000:
	import configparser
else:
	import ConfigParser
	configparser = ConfigParser


class versionclass:
	def call(self):
		return '$(' + self.var + ')'
	def library(self):
		return self.call() + '/' + libName
	def sharedLibrary(self):
		return self.call() + '/' + sharedlibName
	def objects(self):
		return '$(' +self.var + '_OBJS)'
	def path(self):
		return '_' + self.var
	def coinLibrary(self):
		return self.call() + '/' + coinLibName
	def coinSharedLibrary(self):
		return self.call() + '/' + coinSharedLibName
	def coinName(self):
		return coinLibName[3:coinLibName.find('.')]
	def coinSharedName(self):
		return coinSharedLibName[3:coinSharedLibName.find('.')]
	def coinObjects(self):
		return '$(' +self.var + '_COIN_OBJS)'

def bailout(msg):
	print(msg)
	print('Please use the original makeMakefile.config as a template')
	sys.exit()

def loadConfig(sect, key, noError = False):
	if config.has_option(sect, key):
		v = config.get(sect, key)
		print('   [', sect, ']', key, '=', v)
		return v
	else:
		if noError:
			return None
		else:
			bailout('Option "' + key + '" in section "' + sect + '" is missing')

def checkSolver(name, defSolver, extSolvers):
	if(name == defSolver):
		return True
	for s in extSolvers:
		s = s.strip()
		if(name == s):
			return True
	return False


#########################################################
# LOAD CONFIGURATION

config = configparser.ConfigParser()
print('Loading makeMakefile.config...')

try:
	config.readfp(open('makeMakefile.config'))
except IOError:
	bailout('makeMakefile.config not found')

if not config.has_section('GENERAL'):
	bailout('Section "GENERAL" is missing')
if not config.has_section('OGDF'):
	bailout('Section "OGDF" is missing')
if not config.has_section('VERSIONS'):
	bailout('Section "VERSIONS" is missing')
if not config.has_section('COIN'):
	bailout('Section "COIN" is missing')

compilerCommand = loadConfig('GENERAL', 'compilerCommand')
compilerFlags = loadConfig('GENERAL', 'compilerParams')
libCommand = loadConfig('GENERAL', 'libCommand')
sharedlibCommand = loadConfig('GENERAL', 'sharedlibCommand')
rmCommand = loadConfig('GENERAL', 'rmCommand')
mkdirCommand = loadConfig('GENERAL', 'mkdirCommand')
ranlibCommand = loadConfig('GENERAL', 'ranlibCommand', True)
installPrefix = loadConfig('GENERAL', 'installPrefix', True)

sharedLib = loadConfig('GENERAL', 'sharedLib').startswith('t')
libName = loadConfig('OGDF', 'libName')
sharedlibName = loadConfig('OGDF', 'sharedlibName')
includeLegacyCode = loadConfig('OGDF', 'includeLegacyCode').startswith('t')
memoryManager = loadConfig('OGDF', 'memoryManager', True)

# defines for auto generated config_autogen.h header file
config_defines = ''

gccMessageLength = loadConfig('GENERAL', 'gccMessageLength', True)
if gccMessageLength:
	compilerFlags = '-fmessage-length=' + gccMessageLength + ' ' + compilerFlags

libs = ''

if sharedLib:
	config_defines += '#define OGDF_DLL\n'
	if sys.platform == 'win32' or sys.platform == 'cygwin':
		libs += ' -lpsapi'
	else:
		compilerFlags += ' -fPIC'

if memoryManager:
	config_defines += '#define ' + memoryManager + '\n'

addIncludes = ''
useCoin = loadConfig('COIN', 'useCoin').startswith('t')
addOsiCpx = False
addOsiGrb = False
if useCoin:
	shutil.copyfile('config/coinstuff/config.h', 'include/coin/config.h')
	coinLibName = loadConfig('COIN', 'libName')
	coinSharedLibName = loadConfig('COIN', 'sharedlibName')

	defaultSolver   = loadConfig('COIN', 'defaultSolver')
	externalSolvers = loadConfig('COIN', 'externalSolvers').split(';')
	solverIncludes  = loadConfig('COIN', 'solverIncludes').split(';')
	solverLDFlags   = loadConfig('COIN', 'solverLDFlags')

	addOsiCpx = checkSolver('CPX', defaultSolver, externalSolvers)
	addOsiGrb = checkSolver('GRB', defaultSolver, externalSolvers)

	config_defines += '#define USE_COIN\n'
	config_defines += '#define COIN_OSI_' + defaultSolver + '\n'

	for s in externalSolvers:
		s = s.strip();
		if s != '':
			config_defines += '#define OSI_' + s.strip() + '\n'

	for p in solverIncludes:
		p = p.strip()
		if p != '':
			addIncludes += '-I' + p + ' '

	if sharedLib and (sys.platform == 'win32' or sys.platform == 'cygwin'):
		libs += ' -l' + v.coinSharedName()

ogdfFlags = '-I./include ' + addIncludes
coinFlags = '$(COIN_INSTALL_DEFINES) -I./include/coin ' + addIncludes

if sharedLib:
	ogdfFlags += ' -DOGDF_INSTALL'

versions = []
V = config.items('VERSIONS')
if len(V) == 0:
	bailout('Versions missing')
else:
	for ve in V:
		v = versionclass()
		v.var, v.params = ve
		print('   [ VERSIONS ] Name:', v.var, ', Cmd:',v.params)
		versions.append(v)

print('Resulting compiler call (OGDF):', ' '.join([compilerCommand, compilerFlags, ogdfFlags]))
if useCoin:
	print('Resulting compiler call (COIN):', ' '.join([compilerCommand, compilerFlags, coinFlags]))

print('Finished loading makeMakefile.config')

#########################################################
# ANALYZE & GENERATE

print('Generating config_autogen.h ...')
config_autogen = open('include/ogdf/internal/config_autogen.h','w')
config_autogen.write('//\n')
config_autogen.write('// This file has been automatically generated by makeMakefile.py\n')
config_autogen.write('//\n')
config_autogen.write('// Do not change this file manually, instead reconfigure OGDF!\n')
config_autogen.write('\n')
config_autogen.write(config_defines)
config_autogen.close()

if installPrefix:
	print('Generating ogdf.pc ...')
	pc = open('ogdf.pc', 'w')
	pc.write('prefix=' + installPrefix + '\n')
	pc.write('exec_prefix=${prefix}\n')
	pc.write('includedir=${prefix}/include\n')
	pc.write('libdir=${exec_prefix}/lib\n\n')
	pc.write('Name: OGDF\n')
	pc.write('Description: Open Graph Drawing Framework\n')
	pc.write('URL: http://ogdf.net/\n')
	pc.write('Version: ')
	version_h = open('include/ogdf/internal/version.h')
	for line in version_h:
		if line.startswith('#define OGDF_VERSION '):
			pc.write(line[22:line.find('"', 22)])
	version_h.close()
	pc.write('\n')
	pc.write('Cflags: -I${includedir}\n')
	pc.write('Libs: -L${libdir} -lOGDF')
	if useCoin:
		if sharedLib:
			pc.write(' -l' + v.coinSharedName())
		else:
			pc.write(' -l' + v.coinName())
	pc.write('\n')
	pc.close()

print('Analyzing sources & generating Makefile...')

makefile = open('Makefile','w')

# add header
header = open('Makefile.header')
headercontent = header.read()
header.close()
makefile.write(headercontent)

# define general variables

makefile.write('CC = ' + compilerCommand + '\n')
makefile.write('CXXFLAGS = ' + compilerFlags + '\n')
makefile.write('OGDFFLAGS = ' + ogdfFlags + '\n')
makefile.write('COINFLAGS = ' + coinFlags + '\n')
if ranlibCommand:
	makefile.write('RANLIB = ' + ranlibCommand + '\n')
makefile.write('RM = ' + rmCommand + '\n')
makefile.write('AR = ' + libCommand + '\n')
makefile.write('LD = ' + sharedlibCommand + '\n')
makefile.write('MKDIR = ' + mkdirCommand + '\n')

# define release & debug

for v in versions:
	makefile.write(v.var + ' = ' + v.path() + '\n')
	makefile.write('CXXFLAGS_' + v.var + ' = ' + v.params + '\n')
makefile.write('\n');

# just the def. nothing happens yet
def Walk(curdir):
	objs = []
	names = os.listdir(curdir)
	names.sort()

	for name in names:
		# OGDF ignores
		if name.startswith('.') or name.startswith('_') or (name=='legacy' and not includeLegacyCode):
			continue
		# COIN ignores
		if (name == 'OsiCpxSolverInterface.cpp' or name == 'OsiCpxSolverInterface.hpp') and not addOsiCpx:
			continue
		if (name == 'OsiGrbSolverInterface.cpp' or name == 'OsiGrbSolverInterface.hpp') and not addOsiGrb:
			continue

		fullname = posixpath.normpath(posixpath.join(curdir, name))

		if os.path.isdir(fullname) and not os.path.islink(fullname):
			if name != 'abacus' or useCoin == True:
				objs = objs + Walk(fullname)
		else:
			for pat in [ '*.c', '*.cpp' ]:
				if fnmatch.fnmatch(name, pat):
					objfullname = fullname[:-len(pat)+2] + 'o'
					objs.append(objfullname)

					targetAndDepend = subprocess.check_output(' '.join([compilerCommand, compilerFlags, ogdfFlags, '-MM', fullname]), shell=True)
					for v in versions:
						# print target&depend: add full path spec, incl. version & ignore extra line
						path = v.call() + '/' +fullname[:-len(name)]
						makefile.write(path + targetAndDepend[:-1] + '\n')

						# ensure folder
						makefile.write('\t$(MKDIR) ' + v.call() + '/' + fullname[:-len(name)-1] + '\n')
						# what to do: call the compiler
						makefile.write('\t$(CC) $(CXXFLAGS) $(OGDFFLAGS) $(CXXFLAGS_' + v.var + ') -o $@ -c ' + fullname + '\n\n')

					# pattern found: don't try other suffix
					break
	return objs

def WalkCoin(curdir):
	objs = []
	names = os.listdir(curdir)
	names.sort()

	for name in names:
		# OGDF ignores
		if name.startswith('.') or name.startswith('_') or (name=='legacy' and not includeLegacyCode):
			continue
		# COIN ignores
		if (name == 'OsiCpxSolverInterface.cpp' or name == 'OsiCpxSolverInterface.hpp') and not addOsiCpx:
			continue
		if (name == 'OsiGrbSolverInterface.cpp' or name == 'OsiGrbSolverInterface.hpp') and not addOsiGrb:
			continue

		fullname = posixpath.normpath(posixpath.join(curdir, name))

		if os.path.isdir(fullname) and not os.path.islink(fullname):
			if name != 'abacus' or useCoin == True:
				objs = objs + WalkCoin(fullname)
		else:
			for pat in [ '*.c', '*.cpp' ]:
				if fnmatch.fnmatch(name, pat):
					objfullname = fullname[:-len(pat)+2] + 'o'
					if not name.startswith('unitTest'): #conflict if in library!
						objs.append(objfullname)

					# skip .h dependency check here

					for v in versions:
						path = v.call() + '/' +objfullname
						makefile.write(path + ': ' + fullname + '\n')

						# ensure folder
						makefile.write('\t$(MKDIR) ' + v.call() + '/' + fullname[:-len(name)-1] + '\n')
						# what to do: call the compiler
						makefile.write('\t$(CC) $(CXXFLAGS) $(COINFLAGS) $(CXXFLAGS_' + v.var + ') -o $@ -c ' + fullname + '\n\n')

					# pattern found: don't try other suffix
					break
	return objs

# OGDF
objs = Walk('./src/ogdf')

# List all Objs for use in lib-generation and clear
for v in versions:
	makefile.write(v.objects()[2:-1] + ' = \\\n')
	for o in objs:
		makefile.write('\t' + v.call() + '/' + o + ' \\\n')
	makefile.write('\n')

# Coin
if useCoin:
	objsCoin = WalkCoin('./src/coin')

	# List all Objs for use in lib-generation and clear
	for v in versions:
		makefile.write(v.coinObjects()[2:-1] + ' = \\\n')
		for o in objsCoin:
			makefile.write('\t' + v.call() + '/' + o + ' \\\n')
		makefile.write('\n')

# generate test binary targets

makefile.write('\n#######################################################')
makefile.write('\n# test binary targets\n\n')

objsTest = Walk('./test')

for v in versions:
	libdependencies = ''
	if sharedLib:
		if useCoin:
			libdependencies += ' ' + v.coinSharedLibrary()
		libdependencies += ' ' + v.sharedLibrary()
	else:
		if useCoin:
			libdependencies += ' ' + v.coinLibrary()
		libdependencies += ' ' + v.library()
	makefile.write('test/test-' + v.var + ': ' + ' '.join([v.call() + '/' + x for x in objsTest]) + libdependencies + '\n')
	makefile.write('\t$(CC) -o $@ $^ -pthread -L' + v.call() + ' -lOGDF')
	if useCoin:
		if sharedLib:
			makefile.write(' -l' + v.coinSharedName() + ' ' + solverLDFlags)
		else:
			makefile.write(' -l' + v.coinName() + ' ' + solverLDFlags)
	makefile.write('\n\n')

# generate alls and cleans etc...

for v in versions:
	makefile.write('\n#######################################################')
	makefile.write('\n# all, clean, etc. for ' + v.var + '\n\n')

	makefile.write('.PHONY: ' + v.var + ' install' + v.var + ' clean' + v.var + ' clean' + v.var + '-coin\n\n')

	if sharedLib:
		if useCoin:
			makefile.write(v.coinSharedLibrary() + ': ' + v.coinObjects() + '\n')
			makefile.write('\t$(LD) -shared -o ' + v.coinSharedLibrary() + ' ' + v.coinObjects() + '\n\n')

		makefile.write(v.sharedLibrary() + ': ' + v.objects() + '\n')
		makefile.write('\t$(LD) -shared -o ' + v.sharedLibrary() + ' ' + v.objects() + ' ' + libs + ' $(LIBS)\n\n')

		makefile.write(v.var + ': ' + v.sharedLibrary())
		if useCoin:
			makefile.write(' ' + v.coinSharedLibrary())
		makefile.write(' test/test-' + v.var + '\n\n')

	else:
		if useCoin:
			makefile.write(v.coinLibrary() + ': ' + v.coinObjects() + '\n')
			makefile.write('\t$(AR) -r ' + v.coinLibrary() + ' '  + v.coinObjects() + '\n')
			if ranlibCommand:
				makefile.write('\t$(RANLIB) ' + v.coinLibrary() + '\n')
			makefile.write('\n')

		makefile.write(v.library() + ': ' + v.objects() + '\n')
		makefile.write('\t$(AR) -r ' + v.library() + ' '  + v.objects() + ' $(LIBS)\n')
		if ranlibCommand:
			makefile.write('\t$(RANLIB) ' + v.library() + '\n')
		makefile.write('\n')

		makefile.write(v.var + ': ' + v.library())
		if useCoin:
			makefile.write(' ' + v.coinLibrary())
		makefile.write(' test/test-' + v.var + '\n\n')

	makefile.write('clean' + v.var + ':\n')
#	makefile.write('\t$(RM) ' + v.objects() + ' ' + v.library() + '\n\n')
	makefile.write('\t$(RM) ' + v.call() + ' test/test-' + v.var + '\n\n')

	if useCoin:
		makefile.write('clean' + v.var + '-coin:\n')
		makefile.write('\t$(RM) ' + v.coinLibrary() + ' ' + v.coinSharedLibrary() + ' ' + v.call() +'/src/coin\n\n')

	if installPrefix:
		makefile.write('install' + v.var + ':\n')
		makefile.write('\tinstall -d $(DESTDIR)' + installPrefix + '/lib\n')
		if sharedLib:
			makefile.write('\tinstall -m 0644 ' + v.sharedLibrary() + ' $(DESTDIR)' + installPrefix + '/lib/\n')
		else:
			makefile.write('\tinstall -m 0644 ' + v.library() + ' $(DESTDIR)' + installPrefix + '/lib/\n')
		if useCoin:
			if sharedLib:
				makefile.write('\tinstall -m 0644 ' + v.coinSharedLibrary() + ' $(DESTDIR)' + installPrefix + '/lib/\n')
			else:
				makefile.write('\tinstall -m 0644 ' + v.coinLibrary() + ' $(DESTDIR)' + installPrefix + '/lib/\n')
		makefile.write('\n')

def InstallHeaders(curdir, makefile, installPrefix):
	makefile.write('\tinstall -d $(DESTDIR)' + installPrefix + '/' + curdir + '\n')
	names = os.listdir(curdir)
	for name in names:
		filename = curdir + '/' + name
		if (os.path.isdir(filename)):
			InstallHeaders(filename, makefile, installPrefix)
		elif name.endswith('.h') or name.endswith('.hpp'):
			makefile.write('\tinstall -m 0644 ' + filename + ' $(DESTDIR)' + installPrefix + '/' + curdir + '\n')

if installPrefix:
	makefile.write('.PHONY: install-headers install-pkgconfig\n')
	makefile.write('install: installrelease install-headers install-pkgconfig\n\n')
	makefile.write('install-headers:\n')
	InstallHeaders('include/ogdf', makefile, installPrefix)
	if useCoin:
		InstallHeaders('include/coin', makefile, installPrefix)
	makefile.write('\ninstall-pkgconfig: ogdf.pc\n')
	makefile.write('\tinstall -d $(DESTDIR)' + installPrefix + '/lib/pkgconfig\n')
	makefile.write('\tinstall -m 0644 ogdf.pc $(DESTDIR)' + installPrefix + '/lib/pkgconfig\n')

makefile.write('\ndistclean: clean-doc')
for v in versions:
	makefile.write(' clean' + v.var)
makefile.write('\n\t$(RM) Makefile ogdf.pc include/ogdf/internal/config_autogen.h include/coin/config.h\n')

makefile.close()

print('Makefile generated')


